
-- Creacio de taules

CREATE TABLE Model_Comptador
(
  codi        NUMBER            NOT NULL,
  descripcio  VARCHAR2(50),
  fabricant   VARCHAR2(20)      NOT NULL,
  referencia  VARCHAR2(20)      NOT NULL,
  telematic   CHAR(2)           NOT NULL,
 
  CONSTRAINT PK_Model_Comptador PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT UK_Model_Comptador UNIQUE (fabricant, referencia) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT CK_Model_Comptador CHECK (telematic IN('SI', 'NO'))
)
TABLESPACE ts_taules;


CREATE TABLE Comptador
(
  codi            NUMBER        NOT NULL,
  model           NUMBER        NOT NULL,
  adreca          VARCHAR2(50),
  localitat       VARCHAR2(20),
  provincia       VARCHAR2(20),
  pais            VARCHAR2(20),
  any_fabricacio  NUMBER(4)     NOT NULL,
  estat           VARCHAR2(5) DEFAULT 'ALTA',
  
  CONSTRAINT PK_Comptador PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Comptador_ModelComptador FOREIGN KEY (model) REFERENCES Model_Comptador(codi),
  CONSTRAINT CK_Comptador CHECK (any_fabricacio  BETWEEN 1900 AND 2100)
)
TABLESPACE ts_taules;


CREATE TABLE Client
(
  codi        NUMBER      NOT NULL,
  NIF         CHAR(9)	    NOT NULL,
  nom         VARCHAR2(20),
  Cognom1     VARCHAR2(20),
  Cognom2     VARCHAR2(20),
  adreca      VARCHAR2(50),
  localitat   VARCHAR2(20),
  provincia   VARCHAR2(20),
  pais        VARCHAR2(20),
  telf        NUMBER,
  
  CONSTRAINT PK_Client PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT UK_Client UNIQUE (NIF) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Contracte
(
  codi          NUMBER  NOT NULL,
  client        NUMBER  NOT NULL,
  comptador     NUMBER  NOT NULL,
  potencia_KW   NUMBER	NOT NULL,
  data_inici    DATE,
  data_fi       DATE,

  CONSTRAINT PK_Contracte PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Contracte_Client FOREIGN KEY (client) REFERENCES Client(codi),
  CONSTRAINT FK_Contracte_Comptador FOREIGN KEY (comptador) REFERENCES Comptador(codi),
  CONSTRAINT UK_Contracte UNIQUE (client, comptador) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT CK_Contracte CHECK (data_fi >= data_inici)
)
TABLESPACE ts_taules;


CREATE TABLE Lectura
(
  codi          NUMBER       NOT NULL,
  comptador     NUMBER       NOT NULL,
  data          DATE         NOT NULL,
  lectura_KWh   NUMBER       DEFAULT 0 NOT NULL,
  tipus         VARCHAR2(10) NOT NULL,
  consum_KWh    NUMBER       DEFAULT 0 NOT NULL,
  estat       VARCHAR2(5) DEFAULT 'ALTA',

  CONSTRAINT PK_Lectura PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Lectura_Comptador FOREIGN KEY (comptador) REFERENCES Comptador(codi),
  CONSTRAINT UK_Lectura UNIQUE (comptador, data) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT CK_Lectura CHECK (tipus IN('TELEMATIC', 'PRESENCIAL'))
)
TABLESPACE ts_taules;


CREATE TABLE Central_Produccio
(
  codi               NUMBER         NOT NULL,
  adreca             VARCHAR2(50),
  localitat          VARCHAR2(20),
  provincia          VARCHAR2(20),
  pais               VARCHAR2(20),
  capacitat_KWh      NUMBER         NOT NULL,
  darrera_inspeccio  DATE,
 
  CONSTRAINT PK_Central_Produccio PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT UK_Central_Produccio UNIQUE (adreca, localitat, provincia, pais) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Linia_Comunicacio
(
  codi                NUMBER  NOT NULL,
  capacitat_KWh       NUMBER  NOT NULL,
  longitud_Km         NUMBER,
  
  CONSTRAINT PK_Linia_Comunicacio PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Central_Distribucio
(
  codi               NUMBER         NOT NULL,
  adreca             VARCHAR2(50)   NOT NULL,
  localitat          VARCHAR2(20)   NOT NULL,
  provincia          VARCHAR2(20)   NOT NULL,
  pais               VARCHAR2(20)   NOT NULL,
  capacitat_KWh      NUMBER         NOT NULL,
  darrera_inspeccio  DATE,
  
  CONSTRAINT PK_Central_Distribucio PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT UK_Central_Distribucio UNIQUE (adreca, localitat, provincia, pais) USING INDEX TABLESPACE ts_indexs  
)
TABLESPACE ts_taules;


CREATE TABLE Produccio_Mensual
(
  codi                NUMBER  NOT NULL,
  central_produccio   NUMBER  NOT NULL,
  data                DATE    NOT NULL,
  energia_KWh         NUMBER  NOT NULL,
  produccio_KWh       NUMBER  DEFAULT 0 NOT NULL,

  CONSTRAINT PK_Produccio_Mensual PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Produccio_Comptador FOREIGN KEY (central_produccio) REFERENCES Central_Produccio(codi),
  CONSTRAINT UK_Produccio_Mensual UNIQUE (central_produccio, data) USING INDEX TABLESPACE ts_indexs  
)
TABLESPACE ts_taules;


CREATE TABLE Nuclear
(
  codi                  NUMBER  NOT NULL,
  energia_minima        NUMBER,
  rebuig_radioactiu_Kg  NUMBER,

  CONSTRAINT PK_Nuclear PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Nuclear_CentProd FOREIGN KEY (codi) REFERENCES Central_Produccio(codi)
)
TABLESPACE ts_taules;


CREATE TABLE Carbo
(
  codi            NUMBER  NOT NULL,
  emissio_CO2_Kg  NUMBER,

  CONSTRAINT PK_Carbo PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Carbo_CentProd FOREIGN KEY (codi) REFERENCES Central_Produccio(codi)
)
TABLESPACE ts_taules;


CREATE TABLE Termica
(
  codi            NUMBER  NOT NULL,
  emissio_CO2_Kg  NUMBER,

  CONSTRAINT PK_Termica PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Termica_CentProd FOREIGN KEY (codi) REFERENCES Central_Produccio(codi)
)
TABLESPACE ts_taules;


CREATE TABLE Eolica
(
  codi        NUMBER  NOT NULL,
  num_molins  NUMBER,

  CONSTRAINT PK_Eolica PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Eolica_CentProd FOREIGN KEY (codi) REFERENCES Central_Produccio(codi)
)
TABLESPACE ts_taules;


CREATE TABLE Solar
(
  codi         NUMBER  NOT NULL,
  num_panells  NUMBER,

  CONSTRAINT PK_Solar PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Solar_CentProd FOREIGN KEY (codi) REFERENCES Central_Produccio(codi)
)
TABLESPACE ts_taules;


CREATE TABLE Distribucio
(
  codi                 NUMBER   NOT NULL,
  central_distribucio  NUMBER   NOT NULL,
  comptador            NUMBER   NOT NULL,
  servei_actiu         CHAR(2)	NOT NULL,

  CONSTRAINT PK_Distribucio PRIMARY KEY (central_distribucio, comptador) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Distribucio_CentDist FOREIGN KEY (central_distribucio) REFERENCES Central_Distribucio(codi),
  CONSTRAINT FK_Distribucio_Comptador FOREIGN KEY (comptador) REFERENCES Comptador(codi),
  CONSTRAINT UK_Distribucio UNIQUE (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT CK_Distribucio CHECK (servei_actiu IN('SI', 'NO'))
)
TABLESPACE ts_taules;


CREATE TABLE Connexio
(
  codi                   NUMBER  NOT NULL,
  central_produccio      NUMBER	 NOT NULL,
  linia_comunicacio      NUMBER  NOT NULL,
  central_distribucio    NUMBER  NOT NULL,
  servei_actiu           CHAR(2) NOT NULL,

  CONSTRAINT PK_Connexio PRIMARY KEY (central_produccio, linia_comunicacio, Central_Distribucio) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT FK_Connexio_CentProd FOREIGN KEY (central_produccio) REFERENCES Central_Produccio(codi),
  CONSTRAINT FK_Connexio_LiniaCom FOREIGN KEY (linia_comunicacio) REFERENCES Linia_Comunicacio(codi),
  CONSTRAINT FK_Connexio_CentDist FOREIGN KEY (central_distribucio) REFERENCES Central_Distribucio(codi),
  CONSTRAINT UK_Conexxio UNIQUE (codi) USING INDEX TABLESPACE ts_indexs,
  CONSTRAINT CK_Conexxio CHECK (servei_actiu IN('SI', 'NO'))
)
TABLESPACE ts_taules;
  

CREATE TABLE Est1_Cprod_Consum
(
  central_produccio  NUMBER   NOT NULL,
  consum             NUMBER   DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est1_Cprod_Consum PRIMARY KEY (central_produccio) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Est2_Mitja_Consum_Linia
(
  linia         NUMBER      NOT NULL,
  anyo	        NUMBER(4)   NOT NULL,
  mitja_consum  NUMBER	    DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est2_Mitja_Consum_Linia PRIMARY KEY (linia, anyo) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Est3_Linia_Mes_Carrega
(
  linia         NUMBER  NOT NULL,
  consum        NUMBER	DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est3_linia_mes_carrega PRIMARY KEY (linia) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Est4_Linies_Mes_50
(
  anyo	        NUMBER(4)  NOT NULL,
  percentatge   NUMBER     DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est4_linies_mes_50 PRIMARY KEY (anyo) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Est5_Cprod_Menys_30
(
  anyo	  NUMBER(4)  NOT NULL,
  nombre  NUMBER     DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est5_Cprod_menys_30 PRIMARY KEY (anyo) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Est6_Compt_Mes_Consum
(
  comptador  NUMBER  NOT NULL,
  consum     NUMBER DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est6_Compt_Mes_Consum PRIMARY KEY (comptador) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE Est7_Mitja_Consum_Clients
(
  mitja_consum	NUMBER   DEFAULT 0   NOT NULL,
 
  CONSTRAINT PK_Est7_Mitja_Consum_Clients PRIMARY KEY (mitja_consum) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;


CREATE TABLE logs
(
  codi            NUMBER         NOT NULL,
  data_hora    	  VARCHAR2(25)  NOT NULL,
  nom_procediment VARCHAR2(35)  NOT NULL,
  param_entrada	  VARCHAR2(200)  NOT NULL,
  param_sortida	  VARCHAR2(200)  NOT NULL,
 
  CONSTRAINT PK_logs PRIMARY KEY (codi) USING INDEX TABLESPACE ts_indexs
)
TABLESPACE ts_taules;





  
  
