create or replace PROCEDURE SP_ALTA_EST3 (
   p_linia             IN EST3_LINIA_MES_CARREGA.linia%TYPE,
   p_consum            IN EST3_LINIA_MES_CARREGA.consum%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_EST3
   Propsit:            Afegeix una entrada a la taula estadstica EST3_LINIA_MES_CARREGA. 
                        La informaci introduda correspon a la lnia de comunicaci 
                        que ha estat ms carregada histricament a nivell denergia 
                        consumida. s a dir, la lnia que ha transportat ms energia 
                        histricament.
                   
   Parmetres entrada:  p_linia, p_consum            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
err_msg  VARCHAR2 (500);
camp_nul VARCHAR2(20);
E_NUL     EXCEPTION;


BEGIN

   param_entrada:= 'p_linia: '||p_linia||
                 ', p_consum: '||p_consum;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_linia IS NULL) THEN 
     camp_nul:= 'linia';
     RAISE E_NUL;
   ELSIF (p_consum IS NULL) THEN 
     camp_nul:= 'consum';
     RAISE E_NUL;      
   END IF;
 

  -- inserta la nova entrada a la taula EST3
   INSERT INTO EST3_LINIA_MES_CARREGA (linia, consum) VALUES (p_linia, p_consum);
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST3', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST3 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST3', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST3: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST3', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_ALTA_EST3;