create or replace PROCEDURE SP_MODIFICACIO_EST3 
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST3
   Propsit:            Actualitza una entrada a la taula estadstica EST3_LINIA_MES_CARREGA. 
                        La informaci actualitzada correspon a la lnia de comunicaci 
                        que ha estat ms carregada histricament a nivell denergia 
                        consumida. s a dir, la lnia que ha transportat ms energia 
                        histricament.
                   
   Parmetres entrada:  sense parmetres            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
linia_est      NUMBER;
consum_est     NUMBER;
E_NUL          EXCEPTION;


BEGIN

   param_entrada:= 'sense parmetres';

   

  --Calcula la linea ms carregada i el seu consum
   SELECT * 
   INTO linia_est, consum_est
   FROM (SELECT con.linia_comunicacio, SUM(lec.consum_kwh)
         FROM LECTURA lec JOIN DiSTRIBUCIO dist ON lec.comptador = dist.comptador
         JOIN CONNEXIO con ON dist.central_distribucio = con.central_distribucio
         WHERE dist.servei_actiu='SI' AND con.servei_actiu='SI'
         GROUP BY con.linia_comunicacio ORDER BY 2 DESC)
   WHERE ROWNUM =1; 
 
 
   
    -- actualitza cas de que es superi el consum de la lnea s carregada.
    UPDATE EST3_LINIA_MES_CARREGA SET linia = linia_est,
                                      consum = consum_est;    

  
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST3', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION      
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST3: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST3', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST3;