create or replace PROCEDURE SP_MODIFICACIO_EST4 (
   p_any IN Est4_Linies_Mes_50.anyo%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST4
   Propsit:            Actualitza una entrada a la taula estadstica EST4_LINIES_MES_50. 
                        La informaci actualitzada correspon al percentatge de lnies de 
                        comunicaci que superen el 50% denergia consumida respecte la 
                        seva crrega mxima anual donat un any concret.  
                   
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
num_superen    NUMBER;
num_total      NUMBER;
percnt         NUMBER;
E_NUL          EXCEPTION;


BEGIN

   param_entrada:= 'p_any: '||p_any;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_any IS NULL) THEN 
     camp_nul:= 'any';
     RAISE E_NUL;    
   END IF;
 
  --Calcula el nombre de linies que superen el 50% de consum respecte de la seva capacitat
   SELECT COUNT (*) 
   INTO num_superen
   FROM (SELECT con.linia_comunicacio, lin.capacitat_kwh*12 "capacitat_anual",  SUM(lec.consum_kwh)
         FROM LECTURA lec JOIN DiSTRIBUCIO dist ON lec.comptador = dist.comptador
         JOIN CONNEXIO con ON dist.central_distribucio = con.central_distribucio
         JOIN LINIA_COMUNICACIO lin ON con.linia_comunicacio = lin.codi
         WHERE dist.servei_actiu='SI' AND con.servei_actiu='SI' AND EXTRACT(year FROM lec.data)= p_any
         GROUP BY con.linia_comunicacio, lin.capacitat_kwh
         HAVING SUM(lec.consum_kwh) >= lin.capacitat_kwh*12*0.5 );  
 
  --Calcula el nombre de total de linies 
   SELECT COUNT (*)
   INTO num_total
   FROM LINIA_COMUNICACIO;
 
  --Calcula el percentatge de linies que superen el 50%
   percnt:= ROUND( (num_superen/num_total) * 100 , 2);
   
  --Actualitza l'entrada a la taula EST4
   UPDATE EST4_LINIES_MES_50 
   SET percentatge = percnt 
   WHERE anyo = p_any;
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST4', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST4 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST4', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST4: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST4', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST4;