create or replace PROCEDURE SP_MODIFICACIO_EST5 (
   p_any IN EST5_CPROD_MENYS_30.anyo%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST5
   Propsit:            Actualitza una entrada a la taula estadstica EST5_CPROD_MENYS_30. 
                        La informaci actualitzada correspon al nombre de centrals 
                        de producci que generen menys del 30% denergia respecte 
                        la seva capacitat mxima de producci anual donat un any 
                        concret.  
                   
   Parmetres entrada:  p_any            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
num_centrals   NUMBER;
E_NUL          EXCEPTION;


BEGIN

   param_entrada:= 'p_any: '||p_any;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_any IS NULL) THEN 
     camp_nul:= 'any';
     RAISE E_NUL;    
   END IF;
 
  --Calcula el nombre de centrals de producci que generen mensy del 30% de produccio respecte de la seva capacitat
   SELECT COUNT (*)
   INTO num_centrals   
   FROM (SELECT SUM(pm.produccio_kwh)
         FROM CENTRAL_PRODUCCIO cp JOIN PRODUCCIO_MENSUAL pm ON cp.codi = pm.central_produccio
         WHERE EXTRACT(year FROM pm.data)= p_any
         GROUP BY cp.codi, cp.capacitat_kwh
         HAVING SUM (pm.produccio_kwh) < cp.capacitat_kwh*12*0.3);  
 
   
  --actualitza l'entrada a la taula EST5
   UPDATE EST5_CPROD_MENYS_30 
   SET nombre = num_centrals
   WHERE anyo = p_any;
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST5', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST5 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST5', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST5: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST5', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST5;