create or replace PROCEDURE SP_ALTA_EST6(
   p_comptador             IN EST6_COMPT_MES_CONSUM.comptador%TYPE,
   p_consum                IN EST6_COMPT_MES_CONSUM.consum%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_EST6
   Propsit:            Afegeix una entrada a la taula estadstica EST6_COMPT_MES_CONSUM. 
                        La informaci afegida correspon als 10 comptadors que histricament 
                        han tingut ms consum.
                   
   Parmetres entrada:  p_comptador, p_consum           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
err_msg  VARCHAR2 (500);
camp_nul VARCHAR2(20);
E_NUL     EXCEPTION;


BEGIN

   param_entrada:= 'p_comptador: '||p_comptador||
                 ', p_consum: '||p_consum;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_comptador IS NULL) THEN 
     camp_nul:= 'comptador';
     RAISE E_NUL;
   ELSIF (p_consum IS NULL) THEN 
     camp_nul:= 'consum';
     RAISE E_NUL;      
   END IF;
 

  -- inserta la nova entrada a la taula EST6
   INSERT INTO EST6_COMPT_MES_CONSUM (comptador, consum) VALUES (p_comptador, p_consum);
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST6', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST6 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST6', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST6: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST6', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_ALTA_EST6;