create or replace PROCEDURE SP_MODIFICACIO_EST6
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST6
   Propsit:            Actualitza la taula estadstica EST6_COMPT_MES_CONSUM. 
                        La informaci afegida correspon als 10 comptadors que 
                        histricament han tingut ms consum.
                        
   Parmetres entrada:  sense parmetres                              

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
comptador_est  NUMBER;
consum_est     NUMBER;
E_NUL          EXCEPTION;

type r_cursor IS REF CURSOR;
CUR r_cursor;
BEGIN

   param_entrada:= 'sense parmetres';

  --Esborra les dades de la taula EST6
   DELETE FROM EST6_COMPT_MES_CONSUM;   

 
  --Introdueix els comptadors amb ms consum a la taula:
   INSERT INTO EST6_COMPT_MES_CONSUM (comptador, consum) SELECT * 
                                                         FROM (SELECT comptador, MAX(lectura_kwh) 
                                                         FROM LECTURA
                                                         GROUP BY comptador
                                                         ORDER BY 2 DESC)
                                                         WHERE ROWNUM <= 10;
                                                         
  
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST6', param_entrada, 'RSP: '||RSP);

 
 
 
   COMMIT;
EXCEPTION      
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST6: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST6', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST6;