create or replace PROCEDURE SP_ALTA_EST7(
   p_consum                IN EST6_COMPT_MES_CONSUM.consum%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_EST7
   Propsit:            Afegeix una entrada a la taula estadstica EST7_MITJA_CONSUM_CLIENTS. 
                        La informaci afegida correspon al valor de consum histric mitj produt
                        de tots els clients.
                   
   Parmetres entrada:  p_consum            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
err_msg  VARCHAR2 (500);
camp_nul VARCHAR2(20);
E_NUL     EXCEPTION;


BEGIN

   param_entrada:= 'p_consum: '||p_consum;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_consum IS NULL) THEN 
     camp_nul:= 'consum';
     RAISE E_NUL; 
   END IF;
 

  -- inserta la nova entrada a la taula EST7
   INSERT INTO EST7_MITJA_CONSUM_CLIENTS (mitja_consum) VALUES (p_consum);
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST7', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST7 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST7', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST7: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST7', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_ALTA_EST7;