create or replace PROCEDURE SP_MODIFICACIO_EST7 
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST7
   Propsit:            Actualitza una entrada a la taula estadstica EST7_MITJA_CONSUM_CLIENTS. 
                        La informaci afegida correspon al valor de consum histric mitj de tots 
                        els clients.
                   
   Parmetres entrada:  sense parmetres             

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
consum_total   NUMBER;
num_clients    NUMBER;
mitja          NUMBER;
E_NUL          EXCEPTION;


BEGIN

   param_entrada:= 'sense parmetres';

   

  --Calcula el seu consum total histric produit per tots els clients
   SELECT SUM(consum_kwh)
   INTO consum_total
   FROM LECTURA 
   WHERE estat = 'ALTA';  

  --Calcula el nombre de clients que tenen algun contracte donat d'alta.
   SELECT COUNT (DISTINCT client)
   INTO num_clients
   FROM CONTRACTE
   WHERE data_fi IS NULL;
 
  --Calcula el valor mitj de consum historic de tots els clients
   mitja:= ROUND(consum_total / num_clients, 2);
   
    -- actualitza cas de que es superi el consum de la lnea s carregada.
    UPDATE EST7_MITJA_CONSUM_CLIENTS SET mitja_consum = mitja;    

  
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST7', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION      
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST7: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST7', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST7;