create or replace PROCEDURE SP_ALTA_LOGS (
   p_data_hora       IN LOGS.data_hora%TYPE,
   p_nom_procediment IN LOGS.nom_procediment%TYPE,
   p_param_entrada   IN LOGS.param_entrada%TYPE,
   p_param_sortida   IN LOGS.param_sortida%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_LOGS
   Propsit:            afegeix una entrada a la taula LOGS
                   
   Parmetres entrada:  p_data_hora, p_nom_procediment, p_param_entrada, p_param_sortida            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
E_NUL     EXCEPTION;

PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   param_entrada:= 'p_data_hora: '||p_data_hora||
                 ', p_nom_procediment: '||p_nom_procediment||
                 ', p_param_entrada: '||p_param_entrada||
                 ', p_param_sortida: '||p_param_sortida;

  --verifica que els camps obligatoris no siguin nuls
   IF (p_data_hora IS NULL) THEN 
     camp_nul:= 'data_hora';
     RAISE E_NUL;
   ELSIF (p_nom_procediment IS NULL) THEN 
     camp_nul:= 'nom_procediment';     
     RAISE E_NUL;
   ELSIF (p_param_entrada IS NULL) THEN 
     camp_nul:= 'param_entrada';
     RAISE E_NUL;     
   ELSIF (p_param_sortida IS NULL) THEN 
     camp_nul:= 'param_sortida';
     RAISE E_NUL;     
   END IF;
 

  -- inserta la nova entrada a la taula logs
   INSERT INTO LOGS (data_hora, nom_procediment, param_entrada, param_sortida)
   VALUES (p_data_hora, p_nom_procediment, p_param_entrada, p_param_sortida);
   
  -- finalitza amb xit
   RSP:= 'OK';


   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR LOG: el camp ' || camp_nul || ' no pot ser nul'; 
        INSERT INTO LOGS (data_hora, nom_procediment, param_entrada, param_sortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);

        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR LOG: ' || err_msg;
        INSERT INTO LOGS (data_hora, nom_procediment, param_entrada, param_sortida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);

              
END SP_ALTA_LOGS;