create or replace PROCEDURE SP_ALTA_EST1 (
   p_central_produccio IN CENTRAL_PRODUCCIO.codi%TYPE,
   p_consum            IN NUMBER
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_EST1
   Propsit:            Afegeix una entrada a la taula estadstica EST1_CPROD_CONSUM. 
                        La informaci introduda correspon al consum  histric produt 
                        pels comptadors que depenen duna central de producci determinada. 
                   
   Parmetres entrada:  p_central_produccio, p_consum            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
err_msg  VARCHAR2 (500);
camp_nul VARCHAR2(20);
E_NUL     EXCEPTION;


BEGIN

   param_entrada:= 'p_central_produccio: '||p_central_produccio||
                 ', p_consum: '||p_consum;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_central_produccio IS NULL) THEN 
     camp_nul:= 'central_produccio';
     RAISE E_NUL;
   ELSIF (p_consum IS NULL) THEN 
     camp_nul:= 'consum';
     RAISE E_NUL;      
   END IF;
 

  -- inserta la nova entrada a la taula EST1
   INSERT INTO EST1_CPROD_CONSUM (central_produccio, consum) VALUES (p_central_produccio, p_consum);
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST1', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST1 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST1', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST1: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST1', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_ALTA_EST1;