create or replace PROCEDURE SP_MODIFICACIO_EST1 (
   p_central_produccio IN EST1_CPROD_CONSUM.central_produccio%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST1
   Propsit:            Actualitza una entrada a la taula estadstica EST1_CPROD_CONSUM. 
                        La informaci actualitzada correspon al consum  histric produt 
                        pels comptadors que depenen duna central de producci determinada. 
                   
   Parmetres entrada:  p_central_produccio              

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
consum_total   NUMBER;
E_NUL          EXCEPTION;


BEGIN

   param_entrada:= 'p_central_produccio: '||p_central_produccio;

                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_central_produccio IS NULL) THEN 
     camp_nul:= 'central_produccio';
     RAISE E_NUL;
   END IF;
 
  --Calcula el consum total de central de producci. 
   SELECT NVL(SUM(lec.consum_kwh), 0)
   INTO consum_total
   FROM LECTURA lec JOIN DiSTRIBUCIO dist ON lec.comptador = dist.comptador
   JOIN CONNEXIO con ON dist.central_distribucio = con.central_distribucio
   WHERE con.central_produccio= p_central_produccio AND dist.servei_actiu='SI' 
   AND con.servei_actiu='SI' AND lec.estat='ALTA'; 
   

   
  --Actualitza el consum a la  taula EST1
   UPDATE EST1_CPROD_CONSUM SET consum= consum_total WHERE central_produccio = p_central_produccio;

  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST1', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST1 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST1', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST1: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST1', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST1;