create or replace PROCEDURE SP_ALTA_EST2 (
   p_linia             IN est2_mitja_consum_linia.linia%TYPE,
   p_anyo              IN est2_mitja_consum_linia.anyo%TYPE,
   p_consum            IN est2_mitja_consum_linia.mitja_consum%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_EST2
   Propsit:            Afegeix una entrada a la taula estadstica EST2_MITJA_CONSUM_LINIA. 
                        La informaci introduda correspon al valor mitj de lenergia 
                        consumida pels comptadors que salimenten mitjanant una lnia 
                        de comunicaci determinada durant un any determinat. 
                   
   Parmetres entrada:  p_linia, p_anyo, p_consum            

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP  VARCHAR2 (500);
err_msg  VARCHAR2 (500);
camp_nul VARCHAR2(20);
E_NUL     EXCEPTION;


BEGIN

   param_entrada:= 'p_linia: '||p_linia||
                 ', p_anyo: '||p_anyo||
                 ', p_consum: '||p_consum;


  --verifica que els camps obligatoris no siguin nuls
   IF (p_linia IS NULL) THEN 
     camp_nul:= 'p_linia';
     RAISE E_NUL;
   ELSIF (p_anyo IS NULL) THEN 
     camp_nul:= 'anyo';
     RAISE E_NUL;     
   ELSIF (p_consum IS NULL) THEN 
     camp_nul:= 'consum';
     RAISE E_NUL;      
   END IF;
 

  -- inserta la nova entrada a la taula EST2
   INSERT INTO EST2_MITJA_CONSUM_LINIA (linia, anyo, mitja_consum) VALUES (p_linia, p_anyo, p_consum);
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST2', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST2 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST2', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST2: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EST2', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_ALTA_EST2;