create or replace PROCEDURE SP_MODIFICACIO_EST2 (
   p_linia             IN est2_mitja_consum_linia.linia%TYPE,
   p_anyo              IN est2_mitja_consum_linia.anyo%TYPE
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EST2
   Propsit:            Actualitza una entrada a la taula estadstica EST2_MITJA_CONSUM_LINIA. 
                        La informaci introduda correspon al valor mitj de lenergia 
                        consumida pels comptadors que salimenten mitjanant una lnia 
                        de comunicaci determinada durant un any determinat. 
                   
   Parmetres entrada:  p_linia, p_anyo           

   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
RSP            VARCHAR2 (500);
err_msg        VARCHAR2 (500);
camp_nul       VARCHAR2(20);
num_mesos      NUMBER;
consum_total   NUMBER;
E_NUL          EXCEPTION;


BEGIN

   param_entrada:= 'p_linia: '||p_linia||
                 ', p_anyo: '||p_anyo;
                 

  --verifica que els camps obligatoris no siguin nuls
   IF (p_linia IS NULL) THEN 
     camp_nul:= 'p_linia';
     RAISE E_NUL;
   ELSIF (p_anyo IS NULL) THEN 
     camp_nul:= 'anyo';
     RAISE E_NUL;      
   END IF;
   
  --Calcula el consum total de la linea.
   SELECT SUM(lec.consum_kwh)
   INTO consum_total
   FROM LECTURA lec JOIN DiSTRIBUCIO dist ON lec.comptador = dist.comptador
   JOIN CONNEXIO con ON dist.central_distribucio = con.central_distribucio
   WHERE con.linia_comunicacio= p_linia AND EXTRACT(year FROM lec.data)= p_anyo AND con.servei_actiu='SI';

  --Calcula la suma del nombre de mesos en que s'han enregistrat lectures relacionades amb la lnia de comunicaci
  --per cada comptador.
   SELECT COUNT (COUNT(*))
   INTO num_mesos
   FROM LECTURA lec JOIN DiSTRIBUCIO dist ON lec.comptador = dist.comptador
   JOIN CONNEXIO con ON dist.central_distribucio = con.central_distribucio
   WHERE con.linia_comunicacio= p_linia AND EXTRACT(year FROM lec.data)= p_anyo AND con.servei_actiu='SI'
   GROUP BY lec.comptador, EXTRACT(month from lec.data);
 

  -- actualitza la mitja 
   UPDATE EST2_MITJA_CONSUM_LINIA SET mitja_consum = ROUND(consum_total/num_mesos, 2) WHERE linia=p_linia AND anyo = p_anyo;    

  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST2', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR EST2 el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST2', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR EST2: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EST2', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
              
END SP_MODIFICACIO_EST2;