create or replace PROCEDURE SP_ALTA_MODEL_COMPTADOR (
   p_descripcio IN  MODEL_COMPTADOR.descripcio%TYPE,
   p_fabricant  IN  MODEL_COMPTADOR.fabricant%TYPE,
   p_referencia IN  MODEL_COMPTADOR.referencia%TYPE,
   p_telematic  IN  MODEL_COMPTADOR.telematic%TYPE,
   RSP          OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_MODEL_COMPTADOR
   Propsit:            afegeix un model a la taula MODEL_COMPTADOR 
                   
   Parmetres entrada:   -	p_descripcio VARCHAR2(50): descripci del model.
                         -	p_fabricant VARCHAR2(20): nom del fabricant.
                         -	p_referencia  VARCHAR2(20): referncia del model.
                         -	p_telematic CHAR(2): indica si admet lectura telemtica (SI/NO). 
                                                               
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg  VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
BEGIN
   
   param_entrada:= 'p_descripcio: '||p_descripcio||
                 ', p_fabricant: '||p_fabricant||
                 ', p_referencia: '||p_referencia||
                 ', p_telematic: '||p_telematic;
   
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_fabricant IS NULL) THEN 
     camp_nul:= 'fabricant';
     RAISE E_NUL;
   ELSIF (p_referencia IS NULL) THEN 
     camp_nul:= 'referencia';
     RAISE E_NUL;
   ELSIF (p_telematic IS NULL) THEN 
     camp_nul:= 'telematic';
     RAISE E_NUL; 
   END IF; 

  -- inserta el nou model
   INSERT INTO MODEL_COMPTADOR (descripcio, fabricant, referencia, telematic)
   VALUES (p_descripcio, UPPER(p_fabricant), UPPER(p_referencia), UPPER(p_telematic));
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un model amb mateix: fabricant, referencia';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
          
        ELSIF (SQLCODE = -02290) THEN 
          RSP:= 'ERROR: el valor del camp telematic ha de ser SI/NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);  
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);          
        END IF;
              
END SP_ALTA_MODEL_COMPTADOR;