create or replace PROCEDURE SP_MODIFICACIO_MODEL_COMPTADOR (
   p_codi       IN  MODEL_COMPTADOR.codi%TYPE,
   p_descripcio IN  MODEL_COMPTADOR.descripcio%TYPE,
   p_fabricant  IN  MODEL_COMPTADOR.fabricant%TYPE,
   p_referencia IN  MODEL_COMPTADOR.referencia%TYPE,
   p_telematic  IN  MODEL_COMPTADOR.telematic%TYPE,
   RSP          OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_MODEL_COMPTADOR
   Propsit:            modifica dades d'un model a la taula MODEL_COMPTADOR 
                   
   Parmetres entrada:  p_codi, p_descripcio, p_fabricant, p_referencia, p_telematic
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix          INTEGER;
err_msg           VARCHAR2(200);
camp_nul          VARCHAR2(20);
E_NUL             EXCEPTION;
E_NO_EXISTEIX     EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_descripcio: '||p_descripcio||
                 ', p_fabricant: '||p_fabricant||
                 ', p_referencia: '||p_referencia||
                 ', p_telematic: '||p_telematic;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_fabricant IS NULL) THEN 
     camp_nul:= 'fabricant';
     RAISE E_NUL;
   ELSIF (p_referencia IS NULL) THEN 
     camp_nul:= 'referencia';
     RAISE E_NUL;
   ELSIF (p_telematic IS NULL) THEN 
     camp_nul:= 'telematic';
     RAISE E_NUL; 
   END IF; 

  --verifica que el model existeix
   SELECT COUNT (*)
   INTO existeix
   FROM MODEL_COMPTADOR
   WHERE codi = p_codi ;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   -- modifica les dades del model
   UPDATE MODEL_COMPTADOR 
   SET descripcio = p_descripcio, 
       fabricant = UPPER(p_fabricant),
       referencia = UPPER(p_referencia),
       telematic = UPPER(p_telematic)
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
            
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el model no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
            
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un model amb mateix: fabricant, referencia';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
          
        ELSIF (SQLCODE = -02290) THEN 
          RSP:= 'ERROR: el valor del camp telematic ha de ser SI/NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
          
        ELSE 
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
          
        END IF;
              
END SP_MODIFICACIO_MODEL_COMPTADOR;