create or replace PROCEDURE SP_ALTA_COMPTADOR (
   p_model          IN  COMPTADOR.model%TYPE,
   p_adreca         IN  COMPTADOR.adreca%TYPE,
   p_localitat      IN  COMPTADOR.localitat%TYPE,
   p_provincia      IN  COMPTADOR.provincia%TYPE, 
   p_pais           IN  COMPTADOR.pais%TYPE,
   p_any_fabricacio IN  COMPTADOR.any_fabricacio%TYPE,
   RSP              OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_COMPTADOR
   Propsit:            afegeix un comptador a la taula COMPTADOR 
                   
   Parmetres entrada:  p_model, p_adreca, p_localitat, p_provincia, p_pais,
                        p_any_fabricacio                    
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
E_FK     EXCEPTION;
BEGIN

   param_entrada:= 'p_model: '||p_model||
                 ', p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||                 
                 ', p_any_fabricacio: '||p_any_fabricacio;

  --verifica que els camps obligatoris no siguin nuls
   IF (p_model IS NULL) THEN 
     camp_nul:= 'model';
     RAISE E_NUL;
   ELSIF (p_any_fabricacio IS NULL) THEN 
     camp_nul:= 'any_fabricacio';
     RAISE E_NUL;
   END IF; 

  --verifica que el model existeix
   SELECT COUNT (*)
   INTO existeix
   FROM MODEL_COMPTADOR
   WHERE codi = p_model ;

   IF (existeix = 0) THEN
     RAISE E_FK;
   END IF;

   -- inserta el nou comptador
   INSERT INTO COMPTADOR (model, adreca, localitat, provincia, pais, any_fabricacio)
   VALUES (p_model, p_adreca, p_localitat, p_provincia, p_pais, p_any_fabricacio);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_COMPTADOR', param_entrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_COMPTADOR', param_entrada, 'RSP: '||RSP);
        
   WHEN E_FK THEN
        RSP:= 'ERROR: el model no existeix';        
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_COMPTADOR', param_entrada, 'RSP: '||RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -02290) THEN 
          RSP:= 'ERROR: el valor del camp any_fabricacio no s correcte';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_COMPTADOR', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_COMPTADOR', param_entrada, 'RSP: '||RSP);
        END IF;
              
END SP_ALTA_COMPTADOR;