create or replace PROCEDURE SP_BAIXA_COMPTADOR (
   p_codi IN  COMPTADOR.codi%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_COMPTADOR
   Propsit:            realitza la baixa 'lgica' d'un comptador. Es finalitza
                        el contracte d'aquest comptador amb la data actual 
                        i es fa la baixa lgica de totes les lectures asociades.

                   
   Parmetres entrada:  p_codi
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
codi_cprod    NUMBER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN

   param_entrada:= 'p_codi: '||p_codi;
                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix el comptador
   SELECT COUNT (*)
   INTO existeix
   FROM COMPTADOR
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  --  Es fa la baixa lgica del comptador.
   UPDATE COMPTADOR
   SET estat = 'BAIXA'
   WHERE codi = p_codi;

  --  Es finalitza  el contracte d'aquest comptador amb la data actual.
   UPDATE CONTRACTE
   SET data_fi = SYSDATE
   WHERE comptador = p_codi AND data_fi IS NULL;
   
   
  --  Es fa la baixa lgica de totes les lectures d'aquest comptador.
   UPDATE LECTURA
   SET estat = 'BAIXA'
   WHERE comptador = p_codi; 

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_COMPTADOR', param_entrada, 'RSP: '||RSP);
   
   
   
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST1 ---
  ------------------------------------------------------------------------------
  --Calcula el codi de la central de producci de la que depen el comptador.
   SELECT NVL(SUM(DISTINCT con.central_produccio),0)
   INTO codi_cprod
   FROM DISTRIBUCIO dist JOIN CONNEXIO con ON con.central_distribucio = dist.central_distribucio
   WHERE dist.comptador = p_codi
   AND dist.servei_actiu='SI' AND con.servei_actiu='SI';

  --Actualitza l'entrada de la central de producci a EST1 si el comptador depen d'alguna.
   IF (codi_cprod > 0) THEN
     SP_MODIFICACIO_EST1(codi_cprod);
   END IF;
 
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST7 ---
  ------------------------------------------------------------------------------


  --Actualitza  EST7.
     SP_MODIFICACIO_EST7();

  ------------------------------------------------------------------------------     
   
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_COMPTADOR', param_entrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el comptador no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_COMPTADOR', param_entrada, 'RSP: '||RSP);
         
    
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_COMPTADOR', param_entrada, 'RSP: '||RSP);
              
END SP_BAIXA_COMPTADOR;