create or replace PROCEDURE SP_MODIFICACIO_COMPTADOR (
   p_codi           IN  COMPTADOR.codi%TYPE,
   p_model          IN  COMPTADOR.model%TYPE,
   p_adreca         IN  COMPTADOR.adreca%TYPE,
   p_localitat      IN  COMPTADOR.localitat%TYPE,
   p_provincia      IN  COMPTADOR.provincia%TYPE, 
   p_pais           IN  COMPTADOR.pais%TYPE,
   p_any_fabricacio IN  COMPTADOR.any_fabricacio%TYPE,
   RSP              OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_COMPTADOR
   Propsit:            modifica dades d'un comptador a la taula COMPTADOR 
                   
   Parmetres entrada:  p_codi, p_model, p_adreca, p_localitat, p_provincia, 
                        p_pais, p_any_fabricacio  
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
camp_nul      VARCHAR2(20);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
E_FK          EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_model: '||p_model||
                 ', p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||
                 ', p_any_fabricacio: '||p_any_fabricacio;
                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_model IS NULL) THEN 
     camp_nul:= 'model';  
     RAISE E_NUL;
   ELSIF (p_any_fabricacio IS NULL) THEN 
     camp_nul:= 'any_de_fabricacio';
     RAISE E_NUL;
   END IF; 

  --verifica que el comptador existeix
   SELECT COUNT (*)
   INTO existeix
   FROM COMPTADOR
   WHERE codi = p_codi ;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  --verifica que el model existeix
   SELECT COUNT (*)
   INTO existeix
   FROM MODEL_COMPTADOR
   WHERE codi = p_model ;

   IF (existeix = 0) THEN
     RAISE E_FK;
   END IF;

   -- modifica les dades del comptador
   UPDATE COMPTADOR 
   SET model = p_model, 
       adreca = p_adreca,
       localitat = p_localitat,
       provincia = p_provincia,
       pais = p_pais,
       any_fabricacio = p_any_fabricacio
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
           
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el comptador no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
        
   WHEN E_FK THEN
        RSP:= 'ERROR: el model no existeix';  
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
       
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -02290) THEN 
          RSP:= 'ERROR: el valor del camp any_fabricacio no s correcte';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_MODEL_COMPTADOR', param_entrada, 'RSP: '||RSP);
        END IF;
              
END SP_MODIFICACIO_COMPTADOR;