create or replace PROCEDURE SP_ALTA_CLIENT (
   p_NIF       IN  CLIENT.NIF%TYPE,
   p_nom       IN  CLIENT.nom%TYPE,
   p_cognom1   IN  CLIENT.cognom1%TYPE,
   p_cognom2   IN  CLIENT.cognom2%TYPE,
   p_adreca    IN  CLIENT.adreca%TYPE,
   p_localitat IN  CLIENT.localitat%TYPE,
   p_provincia IN  CLIENT.provincia%TYPE,
   p_pais      IN  CLIENT.pais%TYPE,
   p_telf      IN  CLIENT.telf%TYPE,
   RSP         OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_CLIENT
   Propsit:            afegeix un client a la taula CLIENTS.
                   
   Parmetres entrada:  p_NIF, p_nom,  p_cognom1, p_cognom2, p_adreca, p_localitat,
                        p_provincia, p_pais, p_telf
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg  VARCHAR2(200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
BEGIN

   param_entrada:= 'p_NIF: '||p_NIF||
                 ', p_nom: '||p_nom||
                 ', p_cognom1: '||p_cognom1||
                 ', p_cognom2: '||p_cognom2||
                 ', p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||
                 ', p_telf: '||p_telf;
                 
                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_NIF IS NULL) THEN 
     camp_nul:= 'NIF';
     RAISE E_NUL;
   END IF; 

   -- inserta el nou client
   INSERT INTO CLIENT (NIF, Nom, cognom1, cognom2, adreca, localitat, provincia, pais, telf)
   VALUES (UPPER(p_NIF), p_nom,  p_cognom1, p_cognom2, p_adreca, p_localitat, p_provincia, p_pais, p_telf);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CLIENT', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul';  
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CLIENT', param_entrada, 'RSP: '||RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un client amb mateix: DNI/NIF';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CLIENT', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CLIENT', param_entrada, 'RSP: '||RSP);         
        END IF;
              
END SP_ALTA_CLIENT;