create or replace PROCEDURE SP_BAIXA_CLIENT (
   p_codi IN  CLIENT.codi%TYPE,
   RSP   OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_CLIENT
   Propsit:            Es finalitzen tots els contractes d'aquest client amb 
                        la data actual i es fa la baixa lgica de tots els seus
                        comptadors i lectures asociades.
                   
   Parmetres entrada:  p_codi
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
codi_comptador NUMBER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;

type r_cursor IS REF CURSOR;
CUR r_cursor;
BEGIN

   param_entrada:= 'p_codi: '||p_codi;
                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix el client
   SELECT COUNT (*)
   INTO existeix
   FROM CLIENT
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   
  --  Es fa la baixa lgica de tots els comptadors d'aquest client.
   OPEN CUR FOR SELECT comptador FROM CONTRACTE WHERE client = p_codi;             
                               
   LOOP
     FETCH CUR INTO codi_comptador;
     EXIT WHEN CUR%NOTFOUND;
     SP_BAIXA_COMPTADOR(codi_comptador, RSP);
   END LOOP;  
   CLOSE CUR;  
  


  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CLIENT', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CLIENT', param_entrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el client no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CLIENT', param_entrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CLIENT', param_entrada, 'RSP: '||RSP);
              
END SP_BAIXA_CLIENT;