create or replace PROCEDURE SP_MODIFICACIO_CLIENT (
   p_codi      IN  CLIENT.codi%TYPE,
   p_NIF       IN  CLIENT.NIF%TYPE,
   p_nom       IN  CLIENT.nom%TYPE,
   p_cognom1   IN  CLIENT.cognom1%TYPE,
   p_cognom2   IN  CLIENT.cognom2%TYPE,
   p_adreca    IN  CLIENT.adreca%TYPE,
   p_localitat IN  CLIENT.localitat%TYPE,
   p_provincia IN  CLIENT.provincia%TYPE,
   p_pais      IN  CLIENT.pais%TYPE,
   p_telf      IN  CLIENT.telf%TYPE,
   RSP         OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_CLIENT
   Propsit:            modifica dades d'un client a la taula CLIENT
                   
   Parmetres entrada:  p_codi, p_NIF, p_nom,  p_cognom1, p_cognom2, p_adreca, 
                        p_localitat, p_provincia, p_pais, p_telf
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix          INTEGER;
err_msg           VARCHAR2(200);
camp_nul          VARCHAR2(20);
E_NUL             EXCEPTION;
E_NO_EXISTEIX     EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_NIF: '||p_NIF||
                 ', p_nom: '||p_nom||
                 ', p_cognom1: '||p_cognom1||
                 ', p_cognom2: '||p_cognom2||
                 ', p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||
                 ', p_telf: '||p_telf;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_NIF IS NULL) THEN 
     camp_nul:= 'NIF';
     RAISE E_NUL;     
   END IF; 

  --verifica que el client existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CLIENT
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   -- modifica les dades del client
   UPDATE CLIENT 
   SET NIF = UPPER(p_NIF), 
       nom = p_nom,
       cognom1 = p_cognom1,
       cognom2 = p_cognom2,
       adreca = p_adreca,
       localitat = p_localitat,
       provincia = p_provincia,
       pais = p_pais,
       telf = p_telf
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CLIENT', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CLIENT', param_entrada, 'RSP: '||RSP);
            
   WHEN E_NO_EXISTEIX THEN
        
        RSP:= 'ERROR: el client no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CLIENT', param_entrada, 'RSP: '||RSP);
            
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un client amb mateix: DNI/NIF';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CLIENT', param_entrada, 'RSP: '||RSP);
          
        ELSE 
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CLIENT', param_entrada, 'RSP: '||RSP);
          
        END IF;
              
END SP_MODIFICACIO_CLIENT;