create or replace PROCEDURE SP_ALTA_CONTRACTE (
   p_client      IN  CONTRACTE.client%TYPE,
   p_comptador   IN  CONTRACTE.comptador%TYPE,
   p_potencia_KW IN  CONTRACTE.potencia_KW%TYPE,
   p_data_inici  IN  CONTRACTE.data_inici%TYPE,
   RSP           OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_CONTRACTE
   Propsit:            afegeix un contracte a la taula CONTRACTE
                   
   Parmetres entrada:  p_client, p_comptador, p_potencia_KW, p_data_inici
                        
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2(200);
camp_nul VARCHAR2(20);
ref_err  VARCHAR2(20);
E_NUL    EXCEPTION;
E_FK     EXCEPTION;
E_ALTA   EXCEPTION;
BEGIN
   
   param_entrada:= 'p_client: '||p_client||
                 ', p_comptador: '||p_comptador||
                 ', p_potencia_KW: '||p_potencia_KW||
                 ', p_data_inici: '||p_data_inici;
   
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_client IS NULL) THEN 
     camp_nul:= 'client';
     RAISE E_NUL;
   ELSIF (p_comptador IS NULL) THEN 
     camp_nul:= 'comptador';
     RAISE E_NUL;
   ELSIF (p_potencia_KW IS NULL) THEN 
     camp_nul:= 'potencia_KW';
     RAISE E_NUL; 
   END IF; 

  --verifica que el client existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CLIENT
   WHERE codi = p_client ;

   IF (existeix = 0) THEN
     ref_err:= 'client';
     RAISE E_FK;
   END IF;

  --verifica que el comptador existeix
   SELECT COUNT (*)
   INTO existeix
   FROM COMPTADOR
   WHERE codi = p_comptador ;

   IF (existeix = 0) THEN
     ref_err:= 'comptador';
     RAISE E_FK;
   END IF;


  --verifica que el comptador no estigui assignat a cap altre contracte
   SELECT COUNT (*)
   INTO existeix
   FROM CONTRACTE
   WHERE comptador = p_comptador;

   IF (existeix > 0) THEN
     RAISE E_ALTA;
   END IF;   
   
  -- inserta el nou contracte
   INSERT INTO CONTRACTE (client, comptador, potencia_KW, data_inici)
   VALUES (p_client, p_comptador, p_potencia_KW, p_data_inici);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONTRACTE', param_entrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONTRACTE', param_entrada, 'RSP: '||RSP);
        
    WHEN E_FK THEN
        RSP:= 'ERROR: el ' || ref_err || ' no existeix';    
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONTRACTE', param_entrada, 'RSP: '||RSP);
        
   WHEN E_ALTA THEN
        RSP:= 'ERROR: el comptador ja est assignat a un contracte';    
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LECTURA', param_entrada, 'RSP: '||RSP);          
        
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix un contracte amb mateix: client, comptador';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONTRACTE', param_entrada, 'RSP: '||RSP);
                    
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONTRACTE', param_entrada, 'RSP: '||RSP);
        END IF;
              
END SP_ALTA_CONTRACTE;