create or replace PROCEDURE SP_BAIXA_CONTRACTE (
   p_codi IN  CONTRACTE.codi%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_CONTRACTE
   Propsit:            realitza la baixa 'lgica' d'un contracte. S'assigna 
                        la data actual com data de fi de contracte. Es fa la
                        baixa lgica del comptador associat i es fa la baixa 
                        lgica de totes les lectures asociades.
                   
   Parmetres entrada:  p_codi
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
codi_comptador NUMBER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN

   param_entrada:= 'p_codi: '||p_codi;
         
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix el contracte
   SELECT COUNT (*)
   INTO existeix
   FROM CONTRACTE
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;


  --calcula el comptador
   SELECT comptador
   INTO codi_comptador
   FROM CONTRACTE 
   WHERE codi = p_codi;  

  --  Es fa la baixa lgica del comptador associat.
   SP_BAIXA_COMPTADOR(codi_comptador, RSP);
  
  
  -- realitza la baixa 'lgica' d'un contracte. S'assigna la data actual com
  -- data de fi de contracte.
   UPDATE CONTRACTE
   SET data_fi = SYSDATE
   WHERE codi = p_codi AND data_fi IS NULL;
   
  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CONTRACTE', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CONTRACTE', param_entrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el contracte no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CONTRACTE', param_entrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
       IF (SQLCODE = -02290) THEN 
         RSP:= 'ERROR: data_fi ha de ser posterior a data_inici';
         SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CONTRACTE', param_entrada, 'RSP: '||RSP);
         
       ELSE  
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CONTRACTE', param_entrada, 'RSP: '||RSP);
        END IF;
        
END SP_BAIXA_CONTRACTE;