create or replace PROCEDURE SP_MODIFICACIO_CONTRACTE (
   p_codi        IN  CONTRACTE.codi%TYPE,
   p_potencia_KW IN  CONTRACTE.potencia_KW%TYPE,
   RSP           OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_CONTRACTE
   Propsit:            modifica les dades d'un contracte de la taula CONTRACTE. 
                        Si es vol modificar client o comptador s'ha de fer 
                        un nou contracte.
                   
   Parmetres entrada:  p_codi, p_potencia_KW                   
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
camp_nul      VARCHAR2(20);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_potencia_KW: '||p_potencia_KW;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_potencia_KW IS NULL) THEN 
     camp_nul:= 'potencia_KW';  
     RAISE E_NUL;
   END IF; 

  --verifica que el contracte existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CONTRACTE
   WHERE codi = p_codi ;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  -- NO es verifiquen ni client ni comptador ja que no es permet la modificaci.
  -- Si es vol modificar client o comptador s'ha de fer un nou contracte.
     
   -- modifica les dades del contracte
   UPDATE CONTRACTE 
   SET potencia_KW = p_potencia_KW
   WHERE codi = p_codi;     
        
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CONTRACTE', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CONTRACTE', param_entrada, 'RSP: '||RSP);
           
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: el contracte no existeix';  
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CONTRACTE', param_entrada, 'RSP: '||RSP);
       
   WHEN OTHERS THEN
        ROLLBACK;

        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CONTRACTE', param_entrada, 'RSP: '||RSP);

        
END SP_MODIFICACIO_CONTRACTE;