create or replace PROCEDURE SP_BAIXA_LECTURA (
   p_codi IN  LECTURA.codi%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_LECTURA
   Propsit:            elimina una lectura de la taula LECTURA si
                        es tracta de la lectura ms recent d'un comptador.
                   
   Parmetres entrada:  p_codi
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix          INTEGER;
err_msg           VARCHAR2(500);
codi_comptador    NUMBER;
data_lectura      DATE;
ultima_data       DATE;
codi_cprod        NUMBER;
codi_linia     NUMBER;
any_lectura    NUMBER;
i              INTEGER:=1;
E_NUL             EXCEPTION;
E_NO_EXISTEIX     EXCEPTION;
E_NO_BAIXA        EXCEPTION;

type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN

   param_entrada:= 'p_codi: '||p_codi;
       
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix la lectura
   SELECT COUNT (*)
   INTO existeix
   FROM LECTURA
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   
  --calcula el comptador i data de la lectura existent
   SELECT comptador, data
   INTO codi_comptador, data_lectura
   FROM LECTURA 
   WHERE codi = p_codi;  
   
  --calcula la data de l'ltima lectura del mateix comptador
   SELECT MAX(data)
   INTO ultima_data
   FROM LECTURA
   WHERE comptador = codi_comptador;

  --verifica que es tracta de la lectura ms recent
   IF ( data_lectura <> ultima_data ) THEN
     RAISE E_NO_BAIXA;
   END IF;
   

  -- elimina la lectura
   DELETE FROM LECTURA
   WHERE codi = p_codi;

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_LECTURA', param_entrada, 'RSP: '||RSP);
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST1 ---
  ------------------------------------------------------------------------------

  --Calcula el codi de la central de producci de la que depen el comptador. 
   SELECT DISTINCT con.central_produccio
   INTO codi_cprod
   FROM DISTRIBUCIO dist JOIN CONNEXIO con ON con.central_distribucio = dist.central_distribucio
   WHERE dist.comptador = codi_comptador 
   AND dist.servei_actiu='SI' AND con.servei_actiu='SI';

  --Actualitza l'entrada de la central de producci a EST1.
   SP_MODIFICACIO_EST1(codi_cprod);

  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST2 ---
  ------------------------------------------------------------------------------
   
  --calcula l'any de la lectura
   any_lectura:= EXTRACT(year FROM data_lectura);

  --Calcula el codi de la lnia de comunicaci de la que depen el comptador.
   OPEN CUR FOR
   SELECT DISTINCT con.linia_comunicacio
   FROM DISTRIBUCIO dist JOIN CONNEXIO con ON con.central_distribucio = dist.central_distribucio
   WHERE dist.comptador = codi_comptador AND con.servei_actiu='SI';
      
  --Per cada lnia relacionada:
   LOOP
     FETCH CUR INTO codi_linia;
     EXIT WHEN CUR%NOTFOUND;

      --actualitza la entrada a la taula EST2.
       SP_MODIFICACIO_EST2(codi_linia, any_lectura);

       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST3 ---
  ------------------------------------------------------------------------------
     
  --Actualitza l'entrada de la central de producci a EST3.
   SP_MODIFICACIO_EST3();


  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST4 ---
  ------------------------------------------------------------------------------
     
  --Actualitza l'entrada de l'any a EST4.
   SP_MODIFICACIO_EST4(any_lectura);
   
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST6 ---
  ------------------------------------------------------------------------------
     
  --Actualitza l'entrada de la central de producci a EST6.
   SP_MODIFICACIO_EST6();   
   

  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST7 ---
  ------------------------------------------------------------------------------
     
  --Actualitza l'entrada de la central de producci a EST7.
   SP_MODIFICACIO_EST7();     
------------------------------------------------------------------------------   
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_LECTURA', param_entrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la lectura no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_LECTURA', param_entrada, 'RSP: '||RSP);
  
   WHEN E_NO_BAIXA THEN
        RSP:= 'ERROR: noms es permet esborrar l''ltima lectura d''un comptador';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_LECTURA', param_entrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_LECTURA', param_entrada, 'RSP: '||RSP);
              
END SP_BAIXA_LECTURA;