create or replace PROCEDURE SP_ALTA_CENTRAL_PROD (
   p_adreca            IN  CENTRAL_PRODUCCIO.adreca%TYPE,
   p_localitat         IN  CENTRAL_PRODUCCIO.localitat%TYPE,
   p_provincia         IN  CENTRAL_PRODUCCIO.provincia%TYPE, 
   p_pais              IN  CENTRAL_PRODUCCIO.pais%TYPE,
   p_capacitat_KWh     IN  CENTRAL_PRODUCCIO.capacitat_KWh%TYPE,
   p_darrera_inspeccio IN  CENTRAL_PRODUCCIO.darrera_inspeccio%TYPE,   
   RSP                 OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_CENTRAL_PROD
   Propsit:            afegeix una central de producci a la taula CENTRAL_PRODUCCIO 
                   
   Parmetres entrada:  p_adreca, p_localitat, p_provincia, p_pais,
                        p_capacitat_KWh, p_darrera_inspeccio                    
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
BEGIN


   param_entrada:= 'p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||
                 ', p_capacitat_KWh: '||p_capacitat_KWh||
                 ', p_darrera_inspeccio: '||p_darrera_inspeccio;
                 
                
  --verifica que els camps obligatoris no siguin nuls
   IF (p_capacitat_KWh IS NULL) THEN 
     camp_nul:= 'capacitat_KWh';
     RAISE E_NUL;
   END IF; 

   -- inserta la nova central de produccio
   INSERT INTO CENTRAL_PRODUCCIO (adreca, localitat, provincia, pais, capacitat_KWh, darrera_inspeccio)
   VALUES (p_adreca, p_localitat, p_provincia, p_pais, p_capacitat_KWh, p_darrera_inspeccio);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul';  
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
     
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix una central de producci amb mateix: adreca, localitat, provincia, pais';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
        
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
        END IF;
              
END SP_ALTA_CENTRAL_PROD;