create or replace PROCEDURE SP_BAIXA_CENTRAL_PROD (
   p_codi IN  CENTRAL_PRODUCCIO.codi%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_CENTRAL_PROD
   Propsit:            elimina una central de producci de la taula CENTRAL_PRODUCCIO
                   
   Parmetres entrada:  p_codi
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN

   param_entrada:= 'p_codi: '||p_codi;
                 

  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix la central de producci
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_PRODUCCIO
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  -- elimina la central de produccio
   DELETE FROM CENTRAL_PRODUCCIO
   WHERE codi = p_codi;

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la central de producci no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
        
        IF (SQLCODE = -02292) THEN 
          RSP:= 'ERROR: no es pot esborrar la central perqu est relacionada amb altres elements.';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
        
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
        END IF;
        
END SP_BAIXA_CENTRAL_PROD;