create or replace PROCEDURE SP_MODIFICACIO_CENTRAL_PROD (
   p_codi              IN  CENTRAL_PRODUCCIO.codi%TYPE,
   p_adreca            IN  CENTRAL_PRODUCCIO.adreca%TYPE,
   p_localitat         IN  CENTRAL_PRODUCCIO.localitat%TYPE,
   p_provincia         IN  CENTRAL_PRODUCCIO.provincia%TYPE, 
   p_pais              IN  CENTRAL_PRODUCCIO.pais%TYPE,
   p_capacitat_KWh     IN  CENTRAL_PRODUCCIO.capacitat_KWh%TYPE,
   p_darrera_inspeccio IN  CENTRAL_PRODUCCIO.darrera_inspeccio%TYPE,   
   RSP                 OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_CENTRAL_PROD
   Propsit:            modifica dades d'una central de producci a la taula CENTRAL_PRODUCCIO 
                   
   Parmetres entrada:  p_codi, p_adreca, p_localitat, p_provincia, p_pais,
                        p_capacitat_KWh, p_darrera_inspeccio       
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
camp_nul      VARCHAR2(20);
any_actual    NUMBER;
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||
                 ', p_capacitat_KWh: '||p_capacitat_KWh||
                 ', p_darrera_inspeccio: '||p_darrera_inspeccio;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_capacitat_KWh IS NULL) THEN 
     camp_nul:= 'capacitat_KWh';  
     RAISE E_NUL;
   END IF; 

  --verifica que la central de producci existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_PRODUCCIO
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   -- modifica les dades de la central de producci
   UPDATE CENTRAL_PRODUCCIO 
   SET adreca = p_adreca,
       localitat = p_localitat,
       provincia = p_provincia,
       pais = p_pais,
       capacitat_KWh = p_capacitat_KWh,
       darrera_inspeccio = p_darrera_inspeccio
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
   
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST5 ---
  -- (en cas de modificar la capacitat mxima s'ha d'actualitzar la estadstica. 
  --  Per noms s''acualitzen les dades de l''any en curs).
  ------------------------------------------------------------------------------
       
      -- calcula l'any actual. 
       any_actual:= EXTRACT (year FROM SYSDATE);

      -- calcula si existeix una entrada a la taula per l'any en curs
       SELECT COUNT (*)
       INTO existeix
       FROM EST5_CPROD_MENYS_30
       WHERE anyo = any_actual;

      --Si existeix actualitza la entrada.
       IF (existeix > 0) THEN 
       SP_MODIFICACIO_EST5(any_actual);  
       END IF;


------------------------------------------------------------------------------
   
   
   
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
           
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la central de producci no existeix';  
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
       
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix una central de producci amb mateix: adreca, localitat, provincia, pais';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_PROD', param_entrada, 'RSP: '||RSP);
        END IF;

END SP_MODIFICACIO_CENTRAL_PROD;