create or replace PROCEDURE SP_ALTA_LINIA_COM (
   p_capacitat_KWh IN  LINIA_COMUNICACIO.capacitat_KWh%TYPE,
   p_longitud_Km   IN  LINIA_COMUNICACIO.longitud_Km%TYPE,
   RSP             OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_LINIA_COM
   Propsit:            afegeix una lnia de comunicaci a la taula LINIA_COMUNICACIO
                   
   Parmetres entrada:  p_capacitat_KWh, p_longitud_Km                   
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2 (500);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
BEGIN
   
   param_entrada:= 'p_capacitat_KWh: '||p_capacitat_KWh||
                 ', p_longitud_Km: '||p_longitud_Km;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_capacitat_KWh IS NULL) THEN 
     camp_nul:= 'capacitat_Kwh';
     RAISE E_NUL;
   END IF; 

   -- inserta la nova lnia
   INSERT INTO LINIA_COMUNICACIO (capacitat_KWh, longitud_Km)
   VALUES (p_capacitat_KWh, p_longitud_Km);
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LINIA_COM', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LINIA_COM', param_entrada, 'RSP: '||RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 500);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_LINIA_COM', param_entrada, 'RSP: '||RSP);

              
END SP_ALTA_LINIA_COM;