create or replace PROCEDURE SP_MODIFICACIO_LINIA_COM (
   p_codi           IN  LINIA_COMUNICACIO.codi%TYPE,
   p_capacitat_KWh  IN  LINIA_COMUNICACIO.capacitat_KWh%TYPE,
   p_longitud_Km    IN  LINIA_COMUNICACIO.longitud_Km%TYPE,
   RSP              OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_LINIA_COM
   Propsit:            modifica dades d'una lnia de comunicaci a la taula LINIA_COMUNICACIO 
                   
   Parmetres entrada:  p_codi, p_capacitat_KWh, p_longitud_Km
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
camp_nul      VARCHAR2(20);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
any_actual    NUMBER;

BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_capacitat_KWh: '||p_capacitat_KWh||
                 ', p_longitud_Km: '||p_longitud_Km;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_capacitat_KWh IS NULL) THEN 
     camp_nul:= 'capacitat_KWh';  
     RAISE E_NUL;
   END IF; 

  --verifica que la lnia existeix
   SELECT COUNT (*)
   INTO existeix
   FROM LINIA_COMUNICACIO
   WHERE codi = p_codi ;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   -- modifica les dades de la lnia de comunicaci
   UPDATE LINIA_COMUNICACIO 
   SET capacitat_KWh = p_capacitat_KWh, 
       longitud_Km = p_longitud_Km
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_LINIA_COM', param_entrada, 'RSP: '||RSP);
   
   
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST4 ---
  -- (en cas de modificar la capacitat mxima s'ha d'actualitzar la estadstica. 
  --  Per noms s'acualitzen les dades de l'any en curs, ja que no ha d'afectar
  --  a estadstiques d'anys passats).
  ------------------------------------------------------------------------------
       
      -- calcula l'any actual. 
       any_actual:= EXTRACT (year FROM SYSDATE);

      -- calcula si existeix una entrada a la taula per l'any en curs
       SELECT COUNT (*)
       INTO existeix
       FROM EST4_LINIES_MES_50
       WHERE anyo = any_actual;

      --Si existeix actualitza la entrada.
       IF (existeix > 0) THEN 
       SP_MODIFICACIO_EST4(any_actual);  
       END IF;


------------------------------------------------------------------------------
  
   
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_LINIA_COM', param_entrada, 'RSP: '||RSP);
           
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la lnia de comunicaci no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_LINIA_COM', param_entrada, 'RSP: '||RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_LINIA_COM', param_entrada, 'RSP: '||RSP);
                      
END SP_MODIFICACIO_LINIA_COM;