create or replace PROCEDURE SP_MODIFICACIO_CENTRAL_DIST (
   p_codi              IN  CENTRAL_DISTRIBUCIO.codi%TYPE,
   p_adreca            IN  CENTRAL_DISTRIBUCIO.adreca%TYPE,
   p_localitat         IN  CENTRAL_DISTRIBUCIO.localitat%TYPE,
   p_provincia         IN  CENTRAL_DISTRIBUCIO.provincia%TYPE, 
   p_pais              IN  CENTRAL_DISTRIBUCIO.pais%TYPE,
   p_capacitat_KWh     IN  CENTRAL_DISTRIBUCIO.capacitat_KWh%TYPE,
   p_darrera_inspeccio IN  CENTRAL_DISTRIBUCIO.darrera_inspeccio%TYPE,   
   RSP                 OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_CENTRAL_DIST
   Propsit:            modifica dades d'una central de distribuci a la taula CENTRAL_DISTRIBUCIO
                   
   Parmetres entrada:  p_codi, p_adreca, p_localitat, p_provincia, p_pais,
                        p_capacitat_KWh, p_darrera_inspeccio       
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
camp_nul      VARCHAR2(20);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_adreca: '||p_adreca||
                 ', p_localitat: '||p_localitat||
                 ', p_provincia: '||p_provincia||
                 ', p_pais: '||p_pais||
                 ', p_capacitat_KWh: '||p_capacitat_KWh||
                 ', p_darrera_inspeccio: '||p_darrera_inspeccio;
   
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   ELSIF (p_capacitat_KWh IS NULL) THEN 
     camp_nul:= 'capacitat_KWh';
     RAISE E_NUL;
   ELSIF (p_adreca IS NULL) THEN 
     camp_nul:= 'adreca';
     RAISE E_NUL;       
   ELSIF (p_localitat IS NULL) THEN 
     camp_nul:= 'localitat';
     RAISE E_NUL;     
   ELSIF (p_provincia IS NULL) THEN 
     camp_nul:= 'provincia';
     RAISE E_NUL;    
   ELSIF (p_pais IS NULL) THEN 
     camp_nul:= 'pais';
     RAISE E_NUL;  
   END IF; 

  --verifica que la central de distribuci existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_DISTRIBUCIO
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

   -- modifica les dades de la central de distribuci
   UPDATE CENTRAL_DISTRIBUCIO
   SET adreca = p_adreca,
       localitat = p_localitat,
       provincia = p_provincia,
       pais = p_pais,
       capacitat_KWh = p_capacitat_KWh,
       darrera_inspeccio = p_darrera_inspeccio
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_DIST', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_DIST', param_entrada, 'RSP: '||RSP);
           
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la central de ditribuci no existeix'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_DIST', param_entrada, 'RSP: '||RSP);
       
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix una central de distribuci amb mateix: adreca, localitat, provincia, pais';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_DIST', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_CENTRAL_DIST', param_entrada, 'RSP: '||RSP);
        END IF;

END SP_MODIFICACIO_CENTRAL_DIST;