create or replace PROCEDURE SP_BAIXA_PRODUCCIO_MENS(
   p_codi IN  PRODUCCIO_MENSUAL.codi%TYPE,
   RSP    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_BAIXA_PRODUCCIO_MENS
   Propsit:            elimina una lectura de la taula PRODUCCIO_MENSUAL si
                        es tracta de la lectura ms recent d'una central.
                        (lectura d'energa histrica produda)

                   
   Parmetres entrada:  p_codi
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix               INTEGER;
err_msg                VARCHAR2(200);
codi_central_produccio NUMBER;
data_lectura           DATE;
ultima_data            DATE;
any_produccio          NUMBER;
E_NUL                  EXCEPTION;
E_NO_EXISTEIX          EXCEPTION;
E_NO_BAIXA             EXCEPTION;
BEGIN

   param_entrada:= 'p_codi: '||p_codi;
                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que existeix la lectura
   SELECT COUNT (*)
   INTO existeix
   FROM PRODUCCIO_MENSUAL
   WHERE codi = p_codi;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;

  --calcula la central i la data de la lectura existent
   SELECT central_produccio, data
   INTO codi_central_produccio, data_lectura
   FROM PRODUCCIO_MENSUAL 
   WHERE codi = p_codi;  
      
  --calcula la data de l'ultima lectura de la mateixa central
   SELECT MAX(data)
   INTO ultima_data
   FROM PRODUCCIO_MENSUAL
   WHERE central_produccio = codi_central_produccio;

   --verifica que es tracta de la lectura ms recent
   IF ( data_lectura <> ultima_data ) THEN
     RAISE E_NO_BAIXA;
   END IF;
    
 
  -- elimina la lectura
   DELETE FROM PRODUCCIO_MENSUAL
   WHERE codi = p_codi;

  -- finalitza amb xit   
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_PRODUCCIO_MENS', param_entrada, 'RSP: '||RSP);
   
   
  ------------------------------------------------------------------------------
                    -- ACTUALITZA EST5 ---
  ------------------------------------------------------------------------------
  
    --calcula l'any de la produccio
   any_produccio:= EXTRACT(year FROM data_lectura);
   
  --Actualitza l'entrada de l'any a EST5.
   SP_MODIFICACIO_EST5(any_produccio);
   
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_PRODUCCIO_MENS', param_entrada, 'RSP: '||RSP);
    
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la lectura no existeix';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_PRODUCCIO_MENS', param_entrada, 'RSP: '||RSP);
  
   WHEN E_NO_BAIXA THEN
        RSP:= 'ERROR: noms es permet esborrar l''ltima lectura d''una central';
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_PRODUCCIO_MENS', param_entrada, 'RSP: '||RSP);
    
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_BAIXA_PRODUCCIO_MENS', param_entrada, 'RSP: '||RSP);
              
END SP_BAIXA_PRODUCCIO_MENS;