create or replace PROCEDURE SP_ALTA_EOLICA (
   p_codi                 IN  EOLICA.codi%TYPE,
   p_num_molins           IN  EOLICA.num_molins%TYPE,
   RSP                    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_EOLICA
   Propsit:            afegeix una central a la taula EOLICA
                   
   Parmetres entrada:  p_codi, p_num_molins        
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix INTEGER;
duplicat INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
E_NUL    EXCEPTION;
E_FK     EXCEPTION;
E_DUP    EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_num_molins: '||p_num_molins;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     camp_nul:= 'codi';
     RAISE E_NUL;
   END IF; 

  --verifica que el codi existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_PRODUCCIO
   WHERE codi = p_codi ;

   IF (existeix = 0) THEN
     RAISE E_FK;
   END IF;
  
  --verifica que el codi no pertanyi a un altre central
   SELECT COUNT (*)
   INTO duplicat
   FROM CENTRAL_PRODUCCIO
   WHERE codi = p_codi and codi IN (SELECT codi FROM nuclear
                                    UNION
                                    SELECT codi FROM carbo
                                    UNION
                                    SELECT codi FROM termica
                                    UNION
                                    SELECT codi FROM eolica
                                    UNION
                                    SELECT codi FROM solar); 

   IF (duplicat > 0) THEN
     RAISE E_DUP;
   END IF;


  -- inserta el la nova central
   INSERT INTO EOLICA (codi, num_molins )
   VALUES (p_codi, p_num_molins );
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EOLICA', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EOLICA', param_entrada, 'RSP: '||RSP);
               
   WHEN E_FK THEN
        RSP:= 'ERROR: el codi no existeix';        
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EOLICA', param_entrada, 'RSP: '||RSP);
        
   WHEN E_DUP THEN
        RSP:= 'ERROR: el codi ja est assignat a un altre central';        
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EOLICA', param_entrada, 'RSP: '||RSP);
     
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_EOLICA', param_entrada, 'RSP: '||RSP);
              
END SP_ALTA_EOLICA;