create or replace PROCEDURE SP_MODIFICACIO_EOLICA (
   p_codi                 IN  EOLICA.codi%TYPE,
   p_num_molins           IN  EOLICA.num_molins%TYPE,
   RSP                    OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_MODIFICACIO_EOLICA
   Propsit:            modifica dades d'una central a la taula EOLICA 
                   
   Parmetres entrada:  p_codi, p_num_molins
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix      INTEGER;
err_msg       VARCHAR2(200);
E_NUL         EXCEPTION;
E_NO_EXISTEIX EXCEPTION;
E_FK          EXCEPTION;
BEGIN
   
   param_entrada:= 'p_codi: '||p_codi||
                 ', p_num_molins: '||p_num_molins;
   
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_codi IS NULL) THEN 
     RAISE E_NUL;
   END IF; 

  --verifica que la central existeix
   SELECT COUNT (*)
   INTO existeix
   FROM EOLICA
   WHERE codi = p_codi ;

   IF (existeix = 0) THEN
     RAISE E_NO_EXISTEIX;
   END IF;


  -- modifica les dades de la central
   UPDATE EOLICA 
   SET num_molins = p_num_molins
   WHERE codi = p_codi;
     
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EOLICA', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp codi no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EOLICA', param_entrada, 'RSP: '||RSP);
           
   WHEN E_NO_EXISTEIX THEN
        RSP:= 'ERROR: la central no existeix'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EOLICA', param_entrada, 'RSP: '||RSP);
       
   WHEN OTHERS THEN
        ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_MODIFICACIO_EOLICA', param_entrada, 'RSP: '||RSP);
              
END SP_MODIFICACIO_EOLICA;