create or replace PROCEDURE SP_ALTA_DISTRIBUCIO (
   p_central_distribucio IN  DISTRIBUCIO.central_distribucio%TYPE,
   p_comptador           IN  DISTRIBUCIO.comptador%TYPE,
   p_servei_actiu        IN  DISTRIBUCIO.servei_actiu%TYPE, 
   RSP                   OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_DISTRIBUCIO
   Propsit:            afegeix una distribuci a la taula DISTRIBUCIO 
                   
   Parmetres entrada:  p_central_distribucio, p_comptador, p_servei_actiu
   
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
ref_err  VARCHAR2(20);
E_NUL    EXCEPTION;
E_FK     EXCEPTION;
BEGIN
   
   param_entrada:= 'p_central_distribucio: '||p_central_distribucio||
                 ', p_comptador: '||p_comptador||
                 ', p_servei_actiu: '||p_servei_actiu;
   
  --verifica que els camps obligatoris no siguin nuls
   IF (p_central_distribucio IS NULL) THEN 
     camp_nul:= 'central_distribucio';
     RAISE E_NUL;
   ELSIF (p_comptador IS NULL) THEN 
     camp_nul:= 'comptador';
     RAISE E_NUL;
   ELSIF (p_servei_actiu IS NULL) THEN 
     camp_nul:= 'servei_actiu';
     RAISE E_NUL;     
   END IF; 

  --verifica que la central de distribuci existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_DISTRIBUCIO
   WHERE codi = p_central_distribucio ;

   IF (existeix = 0) THEN
     ref_err:= 'central_distribucio';   
     RAISE E_FK;
   END IF;
   
  --verifica que el comptador existeix
   SELECT COUNT (*)
   INTO existeix
   FROM COMPTADOR
   WHERE codi = p_comptador ;

   IF (existeix = 0) THEN
     ref_err:= 'comptador';   
     RAISE E_FK;
   END IF;   

   -- inserta la nova distribuci
   INSERT INTO DISTRIBUCIO (central_distribucio, comptador, servei_actiu)
   VALUES (p_central_distribucio, p_comptador, UPPER(p_servei_actiu));
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISTRIBUCIO', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISTRIBUCIO', param_entrada, 'RSP: '||RSP);
        
    WHEN E_FK THEN  
        RSP:= 'ERROR: ' || ref_err || ' no existeix';       
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISTRIBUCIO', param_entrada, 'RSP: '||RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix una distribuci amb mateix: central_distribucio, comptador';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISTRIBUCIO', param_entrada, 'RSP: '||RSP);
          
        ELSIF (SQLCODE = -02290) THEN 
          RSP:= 'ERROR: el valor del camp servei_actiu ha de ser SI/NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISTRIBUCIO', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_DISTRIBUCIO', param_entrada, 'RSP: '||RSP);
          
        END IF;
              
END SP_ALTA_DISTRIBUCIO;