create or replace PROCEDURE SP_ALTA_CONNEXIO (
   p_central_produccio   IN  CONNEXIO.central_produccio%TYPE,
   p_linia_comunicacio   IN  CONNEXIO.linia_comunicacio%TYPE,
   p_central_distribucio IN  CONNEXIO.central_distribucio%TYPE,
   p_servei_actiu        IN  CONNEXIO.servei_actiu%TYPE, 
   RSP                   OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_CONNEXIO
   Propsit:            afegeix una connexi a la taula CONNEXIO 
                   
   Parmetres entrada:  p_central_produccio, p_linia_comunicacio, p_central_distribucio, 
                        p_servei_actiu
   
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
param_entrada  VARCHAR2(500);
existeix INTEGER;
err_msg  VARCHAR2 (200);
camp_nul VARCHAR2(20);
ref_err  VARCHAR2(20);
E_NUL    EXCEPTION;
E_FK     EXCEPTION;
BEGIN
   
   param_entrada:= 'p_central_produccio: '||p_central_produccio||
                 ', p_linia_comunicacio: '||p_linia_comunicacio||
                 ', p_central_distribucio: '||p_central_distribucio||
                 ', p_servei_actiu: '||p_servei_actiu;
                 
  --verifica que els camps obligatoris no siguin nuls
   IF (p_central_produccio IS NULL) THEN 
     camp_nul:= 'central_produccio';
     RAISE E_NUL;
   ELSIF (p_linia_comunicacio IS NULL) THEN 
     camp_nul:= 'linia_comunicacio';     
     RAISE E_NUL;
   ELSIF (p_central_distribucio IS NULL) THEN 
     camp_nul:= 'central_distribucio';
     RAISE E_NUL;     
   ELSIF (p_servei_actiu IS NULL) THEN 
     camp_nul:= 'servei_actiu';
     RAISE E_NUL;     
   END IF; 
   
  --verifica que la central de producci existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_PRODUCCIO
   WHERE codi = p_central_produccio ;   
   
   IF (existeix = 0) THEN
     ref_err:= 'central_produccio';   
     RAISE E_FK;
   END IF;   
   
  --verifica que la lnia de comunicaci existeix
   SELECT COUNT (*)
   INTO existeix
   FROM LINIA_COMUNICACIO
   WHERE codi = p_linia_comunicacio ;

   IF (existeix = 0) THEN
     ref_err:= 'linia_comunicacio';   
     RAISE E_FK;
   END IF;   
   
  --verifica que la central de distribuci existeix
   SELECT COUNT (*)
   INTO existeix
   FROM CENTRAL_DISTRIBUCIO
   WHERE codi = p_central_distribucio ;

   IF (existeix = 0) THEN
     ref_err:= 'central_distribucio';   
     RAISE E_FK;
   END IF;
   
   -- inserta la nova connexi
   INSERT INTO CONNEXIO (central_produccio, linia_comunicacio, central_distribucio, servei_actiu)
   VALUES (p_central_produccio, p_linia_comunicacio, p_central_distribucio, UPPER(p_servei_actiu));
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONNEXIO', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONNEXIO', param_entrada, 'RSP: '||RSP);
        
    WHEN E_FK THEN  
        RSP:= 'ERROR: ' || ref_err || ' no existeix';       
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONNEXIO', param_entrada, 'RSP: '||RSP);
        
   WHEN OTHERS THEN
        ROLLBACK;
        IF (SQLCODE = -00001) THEN 
          RSP:= 'ERROR: ja existeix una connexi amb mateix: central_produccio, linia_comunicacio, central_distribucio';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONNEXIO', param_entrada, 'RSP: '||RSP);
          
        ELSIF (SQLCODE = -02290) THEN 
          RSP:= 'ERROR: el valor del camp servei_actiu ha de ser SI/NO';
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONNEXIO', param_entrada, 'RSP: '||RSP);
          
        ELSE
          err_msg := SUBSTR (SQLERRM, 1, 200);
          RSP:=    'ERROR: ' || err_msg;
          SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_CONNEXIO', param_entrada, 'RSP: '||RSP);
        END IF;
              
END SP_ALTA_CONNEXIO;