
create or replace TYPE TY_R6b AS OBJECT (
  adreca        VARCHAR2(50),
  localitat     VARCHAR2(20),
  provincia     VARCHAR2(20),
  pais          VARCHAR2(20),
  energia_KWh   NUMBER,       
  capacitat_Kwh NUMBER
);
/
   
 
create or replace TYPE TA_R6b AS TABLE OF TY_R6b;             
/


create or replace FUNCTION FU_TOP10_CENT_DIST
RETURN TA_R6b
IS
/******************************************************************************
   Nom:                 FU_TOP10_CENT_DIST
   Propsit:            Llistat de les 10 centraletes de distribuci que 
                        distribueixen ms energia. El clcul es realitza respecte
                        a les dades de consum del mes anterior en el moment de fer
                        la consulta. 
                   
   Parmetres entrada:  -	cap parmetre.
   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna el resultat 
                          de la consulta especificant :
                          	adrea completa de la central de distribuci.
                          	energia emesa durant el mes anterior.
                          	capacitat mxima mensual de la central de distribuci.

                        - 'ERROR: tipus d'error' si ha fracassat         
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg        VARCHAR2 (200);
adreca        VARCHAR2(50);
localitat     VARCHAR2(20);
provincia     VARCHAR2(20);
pais          VARCHAR2(20);
energia_KWh   NUMBER;       
capacitat_Kwh NUMBER;
mes_anterior  NUMBER;
i             INTEGER:=1;
RSP	          VARCHAR2(500);
resultat      TA_R6b:=TA_R6b();
type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN

   param_entrada:= 'sense parmetres';
                 
  --calcula el mes anterior al mes de la data donada
   SELECT EXTRACT(month FROM SYSDATE)-1
   INTO mes_anterior
   FROM dual;
   
   IF (mes_anterior = 0) THEN
     mes_anterior := 12;
   END IF;


   OPEN CUR FOR SELECT * 
                FROM (SELECT cd.adreca, cd.localitat, cd.provincia, cd.pais, SUM(l.consum_kwh ), cd.capacitat_kwh
                      FROM (CENTRAL_DISTRIBUCIO cd JOIN DISTRIBUCIO d ON cd.codi = d.central_distribucio)
                      JOIN LECTURA l ON d.comptador = l.comptador
                      WHERE d.servei_actiu = 'SI' AND EXTRACT(month from data)= mes_anterior AND l.estat='ALTA'
                      AND EXTRACT(year from data)= EXTRACT(year from SYSDATE)
                      GROUP BY cd.adreca, cd.localitat, cd.provincia, cd.pais, cd.capacitat_kwh
                      ORDER BY 5 DESC, provincia, adreca)
                WHERE ROWNUM <= 10;
                
   LOOP
     FETCH CUR INTO adreca, localitat, provincia, pais, energia_kwh, capacitat_kwh;
     EXIT WHEN CUR%NOTFOUND;
       resultat.extend;
       resultat(i):= TY_R6b(adreca, localitat, provincia, pais, energia_kwh, capacitat_kwh);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP10_CENT_DIST', param_entrada, 'RSP: '||RSP);

  --retorna resultat;
   RETURN resultat;
   
EXCEPTION
   WHEN OTHERS THEN
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP10_CENT_DIST', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;
END;