
create or replace TYPE TY_R6c AS OBJECT (
  codi            NUMBER,
  percent_carrega NUMBER,
  ampliable_KWh   NUMBER       
);
/
   
 
create or replace TYPE TA_R6c AS TABLE OF TY_R6c;             
/


create or replace FUNCTION FU_TOP10_LINIES
RETURN TA_R6c
IS
/******************************************************************************
   Nom:                 FU_TOP10_LINIES
   Propsit:            Llistat de les 10 lnies de comunicaci ms carregades 
                        en relaci a la seva prpia capacitat mxima. El clcul 
                        es realitza respecte a les dades de consum del mes anterior 
                        en el moment de fer la consulta.
                   
   Parmetres entrada:  -	cap parmetre.
   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna el resultat 
                          de la consulta especificant :
                          	codi didentificaci de la lnia.
                          	crrega de la lnia entesa com la potncia que suporta 
                            entre la seva capacitat mxima menual.
                          	energia a la que es pot ampliar la lnia de comunicaci.

                        - 'ERROR: tipus d'error' si ha fracassat          
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg        VARCHAR2 (200);
codi_linia    NUMBER;
carrega       NUMBER;
ampliable_Kwh NUMBER;
i             INTEGER:=1;
RSP	          VARCHAR2(500);
mes_anterior  NUMBER;
cprod         NUMBER;
kwh_cprod     NUMBER;
kwh_linies    NUMBER;
resultat      TA_R6c:=TA_R6c();
type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN

   param_entrada:= 'sense parmetres';

  --calcula el mes anterior al mes de la data donada
   SELECT EXTRACT(month FROM SYSDATE)-1
   INTO mes_anterior
   FROM dual;
   
   IF (mes_anterior = 0) THEN
     mes_anterior := 12;
   END IF;

   OPEN CUR FOR SELECT *
                FROM (SELECT lc.codi, ROUND((SUM(l.consum_kwh )*100/lc.capacitat_kwh), 2) 
                      FROM LINIA_COMUNICACIO lc JOIN CONNEXIO c ON lc.codi = c.linia_comunicacio
                      JOIN DISTRIBUCIO d ON c.central_distribucio = d.central_distribucio
                      JOIN LECTURA l ON d.comptador = l.comptador
                      WHERE c.servei_actiu = 'SI' AND d.servei_actiu = 'SI' AND l.estat='ALTA'
                      AND data IN (SELECT data 
                                   FROM LECTURA 
                                   WHERE EXTRACT(month from data)= mes_anterior 
                                   AND EXTRACT(year from data)= EXTRACT(year from SYSDATE))
                      GROUP BY lc.codi, lc.capacitat_kwh
                      ORDER BY 2 DESC)
                WHERE ROWNUM <=10;
   LOOP
     FETCH CUR INTO codi_linia, carrega;
     EXIT WHEN CUR%NOTFOUND;
       resultat.extend;
       
       SELECT DISTINCT central_produccio 
       INTO cprod
       FROM CONNEXIO
       WHERE linia_comunicacio = codi_linia;

       SELECT capacitat_kwh
       INTO kwh_cprod
       FROM CENTRAL_PRODUCCIO
       WHERE codi = cprod;

       SELECT SUM(DISTINCT capacitat_kwh)
       INTO kwh_linies
       FROM  LINIA_COMUNICACIO lc JOIN CONNEXIO c ON c.linia_comunicacio = lc.codi
       WHERE c.central_produccio = cprod;
      
       resultat(i):= TY_R6c(codi_linia, carrega, kwh_cprod - kwh_linies);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP10_LINIES', param_entrada, 'RSP: '||RSP);

  --retorna resultat;
   RETURN resultat;
   
EXCEPTION
   WHEN OTHERS THEN
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_TOP10_LINIES', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;
END;