
create or replace TYPE TY_R6d AS OBJECT (
  NIF             VARCHAR2(9),
  codi_contracte  NUMBER,
  model_comptador NUMBER       
);
/
   
 
create or replace TYPE TA_R6d AS TABLE OF TY_R6d;             
/


create or replace FUNCTION FU_CLIENTS_DISP
RETURN TA_R6d
IS
/******************************************************************************
   Nom:                 FU_CLIENTS_DISP
   Propsit:            El llistat dels clients que disposen de comptadors amb 
                        servei en alta disponibilitat.
                   
   Parmetres entrada:  -	cap parmetre.
   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna el resultat 
                          de la consulta especificant :
                          	DNI/NIF del client.     
                          	codi del contracte. 
                          	codi del model de comptador.

                          NOTA: Si no es troba cap resultat la taula retornada 
                          estar buida.


                        - 'ERROR: tipus d'error' si ha fracassat                
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg        VARCHAR2 (200);
client        VARCHAR2(9);
contracte     NUMBER;
modelc        NUMBER;
i             INTEGER:=1;
RSP	          VARCHAR2(500);
resultat      TA_R6d:=TA_R6d();
type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN

   param_entrada:= 'sense parmetres';


   OPEN CUR FOR SELECT cli.nif, c.codi "CONTRACTE", mc.codi "MODEL COMPTADOR"
                FROM DISTRIBUCIO d JOIN CONTRACTE c ON c.comptador = d.comptador
                JOIN CLIENT cli ON cli.codi = c.client 
                JOIN COMPTADOR compt ON c.comptador = compt.codi
                JOIN MODEL_COMPTADOR mc ON compt.model = mc.codi
                WHERE compt.estat='ALTA'
                GROUP BY d.comptador, c.codi, cli.nif, mc.codi
                HAVING COUNT(*) > 1;                        
   LOOP
     FETCH CUR INTO client, contracte, modelc;
     EXIT WHEN CUR%NOTFOUND;
       resultat.extend;      
       resultat(i):= TY_R6d(client, contracte, modelc);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_CLIENTS_DISP', param_entrada, 'RSP: '||RSP);

  --retorna resultat;
   RETURN resultat;
   
EXCEPTION
   WHEN OTHERS THEN
      --  ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_CLIENTS_DISP', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;
END;