

create or replace TYPE TY_R6e AS OBJECT (
  consum_kwh           NUMBER, 
  energia_produida_kwh NUMBER
);
/
   
 
create or replace TYPE TA_R6e AS TABLE OF TY_R6e;             
/


create or replace FUNCTION FU_CONSUM_TEMP_CENTRAL (
   p_central   IN  CENTRAL_PRODUCCIO.codi%TYPE,
   p_mes_inici IN  NUMBER,
   p_any_inici IN  NUMBER,
   p_mes_fi    IN  NUMBER,
   p_any_fi    IN  NUMBER
)
RETURN TA_R6e
IS
/******************************************************************************
   Nom:                 FU_CONSUM_TEMP_CENTRAL
   Propsit:            Donada una central de producci i un interval de temps, 
                        volem conixer el consum produt pels comptadors que depenen 
                        daquesta central i lenergia produda per la central en aquest 
                        mateix perode. 
                   
   Parmetres entrada:  -	p_central  NUMBER:  codi de la central de producci.
                        -	p_mes_inici  NUMBER:  mes inicial del perode de temps donat.
                        -	p_any_inici  NUMBER:  any inicial del perode de temps donat.
                        -	p_mes_fi  NUMBER:  mes final del perode de temps donat.
                        -	p_any_fi  NUMBER:  any final del perode de temps donat.

   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna el resultat 
                          de la consulta especificant :
                          	consum produt pels comptadors durant el perode de temps.     
                          	energia produda per la central durant el perode de temps. 

                          NOTA: Si no es troba cap resultat la taula retornada 
                          estar buida.

                        - 'ERROR: tipus d'error' si ha fracassat              
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg        VARCHAR2 (200);
camp_nul       VARCHAR2(20);
data_inici     DATE;
data_fi        DATE;
consum         NUMBER; 
energia        NUMBER; 
i              INTEGER:=1;
RSP	           VARCHAR2(500);
resultat       TA_R6e:=TA_R6e();
E_NUL          EXCEPTION;
E_DATA         EXCEPTION;


type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN
   
   param_entrada:= 'p_central: '||p_central||
                 ', p_mes_inici: '||p_mes_inici||
                 ', p_any_inici: '||p_any_inici||
                 ', p_mes_fi: '||p_mes_fi||
                 ', p_any_fi: '||p_any_fi;
                 
  --verifica que els camps no siguin nuls
   IF (p_central IS NULL) THEN 
     camp_nul:= 'central';
     RAISE E_NUL;
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'mes_inici';
     RAISE E_NUL;   
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'any_inici';
     RAISE E_NUL; 
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'mes_fi';
     RAISE E_NUL; 
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'any_fi';
     RAISE E_NUL;      
   END IF;  
   
  --calcula data inici com el primer dia del primer mes/any introduit 
   data_inici:= to_date('01/'||TRIM(p_mes_inici)||'/'||TRIM(p_any_inici),'DD/MM/RR'); 
   
  --calcula data inici com el darrer dia del segon mes/any introduit 
   data_fi:= last_day(to_date('01/'||TRIM(p_mes_fi)||'/'||TRIM(p_any_fi),'DD/MM/RR')); 
   
  --verifica que mes/any final no sigui anterior a mes/any inicial
   IF (data_fi <= data_inici) THEN 
     RAISE E_DATA;
   END IF; 
   
    
   OPEN CUR FOR SELECT NVL(SUM(l.consum_kwh), 0) "CONSUM"
                FROM LECTURA l JOIN  DISTRIBUCIO d ON l.comptador = d.comptador
                JOIN CONNEXIO con ON d.central_distribucio = con.central_distribucio
                WHERE con.central_produccio = p_central
                AND d.servei_actiu = 'SI' AND con.servei_actiu = 'SI'
                AND data BETWEEN data_inici AND data_fi;
                               
   LOOP
     FETCH CUR INTO consum;
     EXIT WHEN CUR%NOTFOUND;
       resultat.extend;
       
       SELECT SUM(energia_kwh) 
       INTO energia
       FROM PRODUCCIO_MENSUAL
       WHERE central_produccio = 1
       AND data BETWEEN to_date('01/'||'6'||'/'||'11','DD/MM/RR') AND last_day(to_date('01/'||'7'||'/'||'11','DD/MM/RR'));

       resultat(i):= TY_R6e(consum, energia);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_CONSUM_TEMP_CENTRAL', param_entrada, 'RSP: '||RSP);

  --retorna resultat;
   RETURN resultat;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;

   WHEN E_DATA THEN
        RSP:= 'ERROR: mes/any inicial ha de ser anterior a mes/any final'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_CONSUM_TEMP_CENTRAL', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;        

   WHEN OTHERS THEN
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_CONSUM_TEMP_CENTRAL', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;
END;