

create or replace TYPE TY_R6f AS OBJECT (
  tipus_lectura   VARCHAR2(20), 
  percentatge     NUMBER
);
/
   
 
create or replace TYPE TA_R6f AS TABLE OF TY_R6f;             
/


create or replace FUNCTION FU_PERCENT_TIPUS_LEC (
   p_mes_inici IN  NUMBER,
   p_any_inici IN  NUMBER,
   p_mes_fi    IN  NUMBER,
   p_any_fi    IN  NUMBER
)
RETURN TA_R6f
IS
/******************************************************************************
   Nom:                 FU_PERCENT_TIPUS_LEC
   Propsit:            Percentatge de lectures de comptadors efectuades de 
                        forma presencial i de forma telemtica en un perode de temps. 
                   
   Parmetres entrada:  -	p_mes_inici  NUMBER:  mes inicial del perode de temps donat.
                        -	p_any_inici  NUMBER:  any inicial del perode de temps donat.
                        -	p_mes_fi  NUMBER:  mes final del perode de temps donat.
                        -	p_any_fi  NUMBER:  any final del perode de temps donat.

   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna el resultat 
                          de la consulta especificant :
                          	tipus de lectura.     
                          	percentatge del tipus de lectura. 

                        - 'ERROR: tipus d'error' si ha fracassat            
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg        VARCHAR2 (200);
camp_nul       VARCHAR2(20);
data_inici     DATE;
data_fi        DATE;
tipus          VARCHAR2(20);
quantitat      NUMBER; 
total          NUMBER; 
i              INTEGER:=1;
RSP	           VARCHAR2(500);
resultat       TA_R6f:=TA_R6f();
E_NUL          EXCEPTION;
E_DATA         EXCEPTION;


type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN
   
   param_entrada:= 'p_mes_inici: '||p_mes_inici||
                 ', p_any_inici: '||p_any_inici||
                 ', p_mes_fi: '||p_mes_fi||
                 ', p_any_fi: '||p_any_fi;
                 
  --verifica que els camps no siguin nuls
   IF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'mes_inici';
     RAISE E_NUL;   
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'any_inici';
     RAISE E_NUL; 
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'mes_fi';
     RAISE E_NUL; 
   ELSIF (p_mes_inici IS NULL) THEN 
     camp_nul:= 'any_fi';
     RAISE E_NUL;      
   END IF;  
   
  --calcula data inici com el primer dia del primer mes/any introduit 
   data_inici:= to_date('01/'||TRIM(p_mes_inici)||'/'||TRIM(p_any_inici),'DD/MM/RR'); 
   
  --calcula data inici com el darrer dia del segon mes/any introduit 
   data_fi:= last_day(to_date('01/'||TRIM(p_mes_fi)||'/'||TRIM(p_any_fi),'DD/MM/RR')); 
   
  --verifica que mes/any final no sigui anterior a mes/any inicial
   IF (data_fi <= data_inici) THEN 
     RAISE E_DATA;
   END IF; 
   
   
  --Calcula la quatitat total de lectures en el perode de temps.
   SELECT COUNT(*)
   INTO total
   FROM LECTURA
   WHERE data BETWEEN data_inici AND data_fi;
   
    
   OPEN CUR FOR SELECT tipus, COUNT(*)
                FROM LECTURA
                WHERE data BETWEEN data_inici AND data_fi
                GROUP BY tipus; 
                               
   LOOP
     FETCH CUR INTO tipus, quantitat;
     EXIT WHEN CUR%NOTFOUND;
       resultat.extend;
       resultat(i):= TY_R6f(tipus, ROUND((quantitat/total)*100, 2));
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_PERCENT_TIPUS_LEC', param_entrada, 'RSP: '||RSP);


  --retorna resultat;
   RETURN resultat;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_PERCENT_TIPUS_LEC', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;

   WHEN E_DATA THEN
        RSP:= 'ERROR: mes/any inicial ha de ser anterior a mes/any final'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_PERCENT_TIPUS_LEC', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;        

   WHEN OTHERS THEN
      --  ROLLBACK;
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_PERCENT_TIPUS_LEC', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;
END;