

create or replace TYPE TY_R6g AS OBJECT (
  codi           NUMBER,
  model          NUMBER,
  adreca         VARCHAR2(50),
  localitat      VARCHAR2(20),
  provincia      VARCHAR2(20),
  pais           VARCHAR2(20),
  any_fabricacio NUMBER(4)  
);
/
   
 
create or replace TYPE TA_R6g AS TABLE OF TY_R6g;             
/


create or replace FUNCTION FU_COMPTADORS_ANTICS (
   p_num_anys IN  NUMBER
)
RETURN TA_R6g
IS
/******************************************************************************
   Nom:                 FU_COMPTADORS_ANTICS
   Propsit:            Llistat de comptadors que superin un determinat nombre 
                        danys dantiguitat.  
                   
   Parmetres entrada:  -	p_num_anys  NUMBER: nombre d'anys d'antiguitat.

   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna el resultat 
                          de la consulta especificant :
                          	codi del comptador.
                          	codi del model del comptador
                          	adrea completa on es troba installat.
                          	any de fabricaci del comptador.

                          NOTA: Si no es troba cap resultat la taula retornada 
                          estar buida.

                        - 'ERROR: tipus d'error' si ha fracassat             
******************************************************************************/
param_entrada  VARCHAR2(500);
err_msg        VARCHAR2 (200);
camp_nul       VARCHAR2(20);
i              INTEGER:=1;
codi           NUMBER;
model          NUMBER;
adreca         VARCHAR2(50);
localitat      VARCHAR2(20);
provincia      VARCHAR2(20);
pais           VARCHAR2(20);
any_fabricacio NUMBER(4);  
RSP	           VARCHAR2(500);
resultat       TA_R6g:=TA_R6g();
E_NUL          EXCEPTION;



type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN

   param_entrada:= 'p_num_anys: '||p_num_anys;

                 
  --verifica que els camps no siguin nuls
   IF (p_num_anys IS NULL) THEN 
     camp_nul:= 'num_anys';
     RAISE E_NUL;     
   END IF;  
   
   
    
   OPEN CUR FOR SELECT codi, model, adreca, localitat, provincia, pais, any_fabricacio
                FROM COMPTADOR
                WHERE any_fabricacio <= EXTRACT( year FROM sysdate - p_num_anys * 365)
                AND estat='ALTA'
                ORDER BY any_fabricacio;
                               
   LOOP
     FETCH CUR INTO codi, model, adreca, localitat, provincia, pais, any_fabricacio;
     EXIT WHEN CUR%NOTFOUND;
       resultat.extend;
       resultat(i):= TY_R6g(codi, model, adreca, localitat, provincia, pais, any_fabricacio);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_COMPTADORS_ANTICS', param_entrada, 'RSP: '||RSP);

  --retorna resultat;
   RETURN resultat;
EXCEPTION
   WHEN E_NUL THEN
        RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_COMPTADORS_ANTICS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;

   WHEN OTHERS THEN
        err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || err_msg;
        SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'FU_COMPTADORS_ANTICS', param_entrada, 'RSP: '||RSP);
        DBMS_OUTPUT.PUT_LINE(RSP);
        return null;
END;